import nonebot
from pathlib import Path
try:
    import ujson as json
except ModuleNotFoundError:
    import json


DRAW_PATH = nonebot.get_driver().config.draw_path

if not DRAW_PATH:
    DRAW_PATH = str(Path("data/draw_card").absolute()) + '/'

_draw_config = Path(rf"{DRAW_PATH}/draw_card_config/draw_card_config.json")


# 开关
PRTS_FLAG = False if str(nonebot.get_driver().config.prts_flag).lower() == 'false' else True
GENSHIN_FLAG = False if str(nonebot.get_driver().config.genshin_flag).lower() == 'false' else True
PRETTY_FLAG = False if str(nonebot.get_driver().config.pretty_flag).lower() == 'false' else True
GUARDIAN_FLAG = False if str(nonebot.get_driver().config.guardian_flag).lower() == 'false' else True
PCR_FLAG = False if str(nonebot.get_driver().config.pcr_flag).lower() == 'false' else True
AZUR_FLAG = False if str(nonebot.get_driver().config.azur_flag).lower() == 'false' else True
FGO_FLAG = False if str(nonebot.get_driver().config.fgo_flag).lower() == 'false' else True
ONMYOJI_FLAG = False if str(nonebot.get_driver().config.onmyoji_flag).lower() == 'false' else True

SEMAPHORE = 5 if not nonebot.get_driver().config.semaphore else int(nonebot.get_driver().config.semaphore)
PCR_TAI = True if str(nonebot.get_driver().config.pcr_tai).lower() == 'true' else False

# 方舟概率
PRTS_SIX_P = 0.02
PRTS_FIVE_P = 0.08
PRTS_FOUR_P = 0.48
PRTS_THREE_P = 0.42

# 原神概率
GENSHIN_FIVE_P = 0.006
GENSHIN_FOUR_P = 0.051
GENSHIN_THREE_P = 0.43
# 保底概率
GENSHIN_G_FIVE_P = 0.016
GENSHIN_G_FOUR_P = 0.13
# 72抽后增加的概率
I72_ADD = 0.0585

# 赛马娘概率
PRETTY_THREE_P = 0.03
PRETTY_TWO_P = 0.18
PRETTY_ONE_P = 0.79

# 坎公骑冠剑
# 角色概率
GUARDIAN_THREE_CHAR_P = 0.0275
GUARDIAN_TWO_CHAR_P = 0.19
GUARDIAN_ONE_CHAR_P = 0.7825
# UP角色
GUARDIAN_THREE_CHAR_UP_P = 0.01375
GUARDIAN_THREE_CHAR_OTHER_P = 0.01375
# 武器概率
GUARDIAN_EXCLUSIVE_ARMS_P = 0.03
GUARDIAN_FIVE_ARMS_P = 0.03
GUARDIAN_FOUR_ARMS_P = 0.09
GUARDIAN_THREE_ARMS_P = 0.27
GUARDIAN_TWO_ARMS_P = 0.58
# UP武器
GUARDIAN_EXCLUSIVE_ARMS_UP_P = 0.01
GUARDIAN_EXCLUSIVE_ARMS_OTHER_P = 0.02

# PCR
PCR_THREE_P = 0.025
PCR_TWO_P = 0.18
PCR_ONE_P = 0.795
# 保底
PCR_G_THREE_P = 0.025
PCR_G_TWO_P = 0.975

# 碧蓝航线
AZUR_FIVE_P = 0.012
AZUR_FOUR_P = 0.07
AZUR_THREE_P = 0.12
AZUR_TWO_P = 0.51
AZUR_ONE_P = 0.3

# FGO
FGO_SERVANT_FIVE_P = 0.01
FGO_SERVANT_FOUR_P = 0.03
FGO_SERVANT_THREE_P = 0.4
FGO_CARD_FIVE_P = 0.04
FGO_CARD_FOUR_P = 0.12
FGO_CARD_THREE_P = 0.4

# 阴阳师
ONMYOJI_SP = 0.0025
ONMYOJI_SSR = 0.01
ONMYOJI_SR = 0.2
ONMYOJI_R = 0.7875


path_dict = {
    'genshin': '原神',
    'prts': '明日方舟',
    'pretty': '赛马娘',
    'guardian': '坎公骑冠剑',
    'pcr': '公主连结',
    'azur': '碧蓝航线',
    'fgo': '命运-冠位指定',
    'onmyoji': '阴阳师',
}

driver: nonebot.Driver = nonebot.get_driver()

config_default_data = {

    'path_dict': {
        'genshin': '原神',
        'prts': '明日方舟',
        'pretty': '赛马娘',
        'guardian': '坎公骑冠剑',
        'pcr': '公主连结',
        'azur': '碧蓝航线',
        'fgo': '命运-冠位指定',
        'onmyoji': '阴阳师',
    },

    'prts': {
        'PRTS_SIX_P': 0.02,
        'PRTS_FIVE_P': 0.08,
        'PRTS_FOUR_P': 0.48,
        'PRTS_THREE_P': 0.42,
    },

    'genshin': {
        'GENSHIN_FIVE_P': 0.006,
        'GENSHIN_FOUR_P': 0.051,
        'GENSHIN_THREE_P': 0.43,
        'GENSHIN_G_FIVE_P': 0.13,
        'GENSHIN_G_FOUR_P': 0.016,
        'I72_ADD': 0.0585,
    },

    'pretty': {
        'PRETTY_THREE_P': 0.03,
        'PRETTY_TWO_P': 0.18,
        'PRETTY_ONE_P': 0.79,
    },

    'guardian': {
        'GUARDIAN_THREE_CHAR_P': 0.0275,
        'GUARDIAN_TWO_CHAR_P': 0.19,
        'GUARDIAN_ONE_CHAR_P': 0.7825,

        'GUARDIAN_THREE_CHAR_UP_P': 0.01375,
        'GUARDIAN_THREE_CHAR_OTHER_P': 0.01375,

        'GUARDIAN_EXCLUSIVE_ARMS_P': 0.03,
        'GUARDIAN_FIVE_ARMS_P': 0.03,
        'GUARDIAN_FOUR_ARMS_P': 0.09,
        'GUARDIAN_THREE_ARMS_P': 0.27,
        'GUARDIAN_TWO_ARMS_P': 0.58,

        'GUARDIAN_EXCLUSIVE_ARMS_UP_P': 0.01,
        'GUARDIAN_EXCLUSIVE_ARMS_OTHER_P': 0.02,
    },

    'pcr': {
        'PCR_THREE_P': 0.025,
        'PCR_TWO_P': 0.18,
        'PCR_ONE_P': 0.795,
    },

    'azur': {
        'AZUR_FIVE_P': 0.012,
        'AZUR_FOUR_P': 0.07,
        'AZUR_THREE_P': 0.12,
        'AZUR_TWO_P': 0.51,
        'AZUR_ONE_P': 0.3,
    },

    'fgo': {
        'FGO_SERVANT_FIVE_P': 0.01,
        'FGO_SERVANT_FOUR_P': 0.03,
        'FGO_SERVANT_THREE_P': 0.4,
        'FGO_CARD_FIVE_P': 0.04,
        'FGO_CARD_FOUR_P': 0.12,
        'FGO_CARD_THREE_P': 0.4,
    },

    'onmyoji': {
        'ONMYOJI_SP': 0.0025,
        'ONMYOJI_SSR': 0.01,
        'ONMYOJI_SR': 0.2,
        'ONMYOJI_R': 0.7875,
    }
}


@driver.on_startup
def check_config():
    global PRTS_SIX_P, PRTS_FOUR_P, PRTS_FIVE_P, PRTS_THREE_P, GENSHIN_G_FIVE_P, config_default_data, \
        GENSHIN_G_FOUR_P, GENSHIN_FOUR_P, GENSHIN_FIVE_P, I72_ADD, path_dict, PRETTY_THREE_P, \
        PRETTY_ONE_P, PRETTY_TWO_P, GENSHIN_THREE_P, GUARDIAN_THREE_CHAR_P, GUARDIAN_TWO_CHAR_P, GUARDIAN_ONE_CHAR_P, \
        GUARDIAN_THREE_CHAR_UP_P, GUARDIAN_THREE_CHAR_OTHER_P, GUARDIAN_EXCLUSIVE_ARMS_P, GUARDIAN_FIVE_ARMS_P, \
        GUARDIAN_FOUR_ARMS_P, GUARDIAN_THREE_ARMS_P, GUARDIAN_TWO_ARMS_P, GENSHIN_FLAG, PRTS_FLAG, \
        PRETTY_FLAG, GUARDIAN_FLAG, GUARDIAN_EXCLUSIVE_ARMS_UP_P, GUARDIAN_EXCLUSIVE_ARMS_OTHER_P, DRAW_PATH, \
        PCR_THREE_P, PCR_TWO_P, PCR_ONE_P, AZUR_FOUR_P, AZUR_THREE_P, AZUR_TWO_P, AZUR_ONE_P, AZUR_FIVE_P, FGO_CARD_FIVE_P,\
        FGO_CARD_FOUR_P, FGO_CARD_THREE_P, FGO_SERVANT_THREE_P, FGO_SERVANT_FOUR_P, FGO_SERVANT_FIVE_P, ONMYOJI_R, ONMYOJI_SP, \
        ONMYOJI_SSR, ONMYOJI_SR
    _draw_config.parent.mkdir(parents=True, exist_ok=True)
    try:
        data = json.load(open(_draw_config, 'r', encoding='utf8'))
    except (FileNotFoundError, ValueError):
        _draw_config.parent.mkdir(parents=True, exist_ok=True)
        json.dump(config_default_data, open(_draw_config, 'w', encoding='utf8'), indent=4, ensure_ascii=False)
        print('draw_card：配置文件不存在或格式错误，已重新生成配置文件.....')
    else:

        try:
            PRTS_SIX_P = float(data['prts']['PRTS_SIX_P'])
            PRTS_FIVE_P = float(data['prts']['PRTS_FIVE_P'])
            PRTS_FOUR_P = float(data['prts']['PRTS_FOUR_P'])
            PRTS_THREE_P = float(data['prts']['PRTS_THREE_P'])
        except KeyError:
            data['prts'] = {}
            data['prts']['PRTS_SIX_P'] = config_default_data['prts']['PRTS_SIX_P']
            data['prts']['PRTS_FIVE_P'] = config_default_data['prts']['PRTS_FIVE_P']
            data['prts']['PRTS_FOUR_P'] = config_default_data['prts']['PRTS_FOUR_P']
            data['prts']['PRTS_THREE_P'] = config_default_data['prts']['PRTS_THREE_P']

        try:
            GENSHIN_FIVE_P = float(data['genshin']['GENSHIN_FIVE_P'])
            GENSHIN_FOUR_P = float(data['genshin']['GENSHIN_FOUR_P'])
            GENSHIN_THREE_P = float(data['genshin']['GENSHIN_THREE_P'])
            GENSHIN_G_FIVE_P = float(data['genshin']['GENSHIN_G_FIVE_P'])
            GENSHIN_G_FOUR_P = float(data['genshin']['GENSHIN_G_FOUR_P'])
            I72_ADD = float(data['genshin']['I72_ADD'])
        except KeyError:
            data['genshin'] = {}
            data['genshin']['GENSHIN_FIVE_P'] = config_default_data['genshin']['GENSHIN_FIVE_P']
            data['genshin']['GENSHIN_FOUR_P'] = config_default_data['genshin']['GENSHIN_FOUR_P']
            data['genshin']['GENSHIN_THREE_P'] = config_default_data['genshin']['GENSHIN_THREE_P']
            data['genshin']['GENSHIN_G_FIVE_P'] = config_default_data['genshin']['GENSHIN_G_FIVE_P']
            data['genshin']['GENSHIN_G_FOUR_P'] = config_default_data['genshin']['GENSHIN_G_FOUR_P']
            data['genshin']['I72_ADD'] = config_default_data['genshin']['I72_ADD']

        try:
            PRETTY_THREE_P = float(data['pretty']['PRETTY_THREE_P'])
            PRETTY_TWO_P = float(data['pretty']['PRETTY_TWO_P'])
            PRETTY_ONE_P = float(data['pretty']['PRETTY_ONE_P'])
        except KeyError:
            data['pretty'] = {}
            data['pretty']['PRETTY_THREE_P'] = config_default_data['pretty']['PRETTY_THREE_P']
            data['pretty']['PRETTY_TWO_P'] = config_default_data['pretty']['PRETTY_TWO_P']
            data['pretty']['PRETTY_ONE_P'] = config_default_data['pretty']['PRETTY_ONE_P']

        try:
            GUARDIAN_THREE_CHAR_P = float(data['guardian']['GUARDIAN_THREE_CHAR_P'])
            GUARDIAN_TWO_CHAR_P = float(data['guardian']['GUARDIAN_TWO_CHAR_P'])
            GUARDIAN_ONE_CHAR_P = float(data['guardian']['GUARDIAN_ONE_CHAR_P'])
            GUARDIAN_THREE_CHAR_UP_P = float(data['guardian']['GUARDIAN_THREE_CHAR_UP_P'])
            GUARDIAN_THREE_CHAR_OTHER_P = float(data['guardian']['GUARDIAN_THREE_CHAR_OTHER_P'])
            GUARDIAN_EXCLUSIVE_ARMS_P = float(data['guardian']['GUARDIAN_EXCLUSIVE_ARMS_P'])
            GUARDIAN_FIVE_ARMS_P = float(data['guardian']['GUARDIAN_FIVE_ARMS_P'])
            GUARDIAN_FOUR_ARMS_P = float(data['guardian']['GUARDIAN_FOUR_ARMS_P'])
            GUARDIAN_THREE_ARMS_P = float(data['guardian']['GUARDIAN_THREE_ARMS_P'])
            GUARDIAN_TWO_ARMS_P = float(data['guardian']['GUARDIAN_TWO_ARMS_P'])
            GUARDIAN_EXCLUSIVE_ARMS_UP_P = float(data['guardian']['GUARDIAN_EXCLUSIVE_ARMS_UP_P'])
            GUARDIAN_EXCLUSIVE_ARMS_OTHER_P = float(data['guardian']['GUARDIAN_EXCLUSIVE_ARMS_OTHER_P'])
        except KeyError:
            data['guardian'] = {}
            data['guardian']['GUARDIAN_THREE_CHAR_P'] = config_default_data['guardian']['GUARDIAN_THREE_CHAR_P']
            data['guardian']['GUARDIAN_TWO_CHAR_P'] = config_default_data['guardian']['GUARDIAN_TWO_CHAR_P']
            data['guardian']['GUARDIAN_ONE_CHAR_P'] = config_default_data['guardian']['GUARDIAN_ONE_CHAR_P']
            data['guardian']['GUARDIAN_THREE_CHAR_UP_P'] = config_default_data['guardian']['GUARDIAN_THREE_CHAR_UP_P']
            data['guardian']['GUARDIAN_THREE_CHAR_OTHER_P'] = config_default_data['guardian']['GUARDIAN_THREE_CHAR_OTHER_P']
            data['guardian']['GUARDIAN_EXCLUSIVE_ARMS_P'] = config_default_data['guardian']['GUARDIAN_EXCLUSIVE_ARMS_P']
            data['guardian']['GUARDIAN_FIVE_ARMS_P'] = config_default_data['guardian']['GUARDIAN_FIVE_ARMS_P']
            data['guardian']['GUARDIAN_FOUR_ARMS_P'] = config_default_data['guardian']['GUARDIAN_FOUR_ARMS_P']
            data['guardian']['GUARDIAN_THREE_ARMS_P'] = config_default_data['guardian']['GUARDIAN_THREE_ARMS_P']
            data['guardian']['GUARDIAN_TWO_ARMS_P'] = config_default_data['guardian']['GUARDIAN_TWO_ARMS_P']
            data['guardian']['GUARDIAN_EXCLUSIVE_ARMS_UP_P'] = config_default_data['guardian']['GUARDIAN_EXCLUSIVE_ARMS_UP_P']
            data['guardian']['GUARDIAN_EXCLUSIVE_ARMS_OTHER_P'] = config_default_data['guardian']['GUARDIAN_EXCLUSIVE_ARMS_OTHER_P']

        try:
            PCR_THREE_P = float(data['pcr']['PCR_THREE_P'])
            PCR_TWO_P = float(data['pcr']['PCR_TWO_P'])
            PCR_ONE_P = float(data['pcr']['PCR_ONE_P'])
        except KeyError:
            data['pcr'] = {}
            data['pcr']['PCR_THREE_P'] = config_default_data['pcr']['PCR_THREE_P']
            data['pcr']['PCR_TWO_P'] = config_default_data['pcr']['PCR_TWO_P']
            data['pcr']['PCR_ONE_P'] = config_default_data['pcr']['PCR_ONE_P']

        try:
            AZUR_FIVE_P = float(data['azur']['AZUR_FIVE_P'])
            AZUR_FOUR_P = float(data['azur']['AZUR_FOUR_P'])
            AZUR_THREE_P = float(data['azur']['AZUR_THREE_P'])
            AZUR_TWO_P = float(data['azur']['AZUR_TWO_P'])
            AZUR_ONE_P = float(data['azur']['AZUR_ONE_P'])
        except KeyError:
            data['azur'] = {}
            data['azur']['AZUR_FIVE_P'] = config_default_data['azur']['AZUR_FIVE_P']
            data['azur']['AZUR_FOUR_P'] = config_default_data['azur']['AZUR_FOUR_P']
            data['azur']['AZUR_THREE_P'] = config_default_data['azur']['AZUR_THREE_P']
            data['azur']['AZUR_TWO_P'] = config_default_data['azur']['AZUR_TWO_P']
            data['azur']['AZUR_ONE_P'] = config_default_data['azur']['AZUR_ONE_P']

        try:
            FGO_SERVANT_FIVE_P = float(data['fgo']['FGO_SERVANT_FIVE_P'])
            FGO_SERVANT_FOUR_P = float(data['fgo']['FGO_SERVANT_FOUR_P'])
            FGO_SERVANT_THREE_P = float(data['fgo']['FGO_SERVANT_THREE_P'])
            FGO_CARD_FIVE_P = float(data['fgo']['FGO_CARD_FIVE_P'])
            FGO_CARD_FOUR_P = float(data['fgo']['FGO_CARD_FOUR_P'])
            FGO_CARD_THREE_P = float(data['fgo']['FGO_CARD_THREE_P'])
        except KeyError:
            data['fgo'] = {}
            data['fgo']['FGO_SERVANT_FIVE_P'] = config_default_data['fgo']['FGO_SERVANT_FIVE_P']
            data['fgo']['FGO_SERVANT_FOUR_P'] = config_default_data['fgo']['FGO_SERVANT_FOUR_P']
            data['fgo']['FGO_SERVANT_THREE_P'] = config_default_data['fgo']['FGO_SERVANT_THREE_P']
            data['fgo']['FGO_CARD_FIVE_P'] = config_default_data['fgo']['FGO_CARD_FIVE_P']
            data['fgo']['FGO_CARD_FOUR_P'] = config_default_data['fgo']['FGO_CARD_FOUR_P']
            data['fgo']['FGO_CARD_THREE_P'] = config_default_data['fgo']['FGO_CARD_THREE_P']

        try:
            ONMYOJI_SP = float(data['onmyoji']['ONMYOJI_SP'])
            ONMYOJI_SSR = float(data['onmyoji']['ONMYOJI_SSR'])
            ONMYOJI_SR = float(data['onmyoji']['ONMYOJI_SR'])
            ONMYOJI_R = float(data['onmyoji']['ONMYOJI_R'])
        except KeyError:
            data['onmyoji'] = {}
            data['onmyoji']['ONMYOJI_SP'] = config_default_data['onmyoji']['ONMYOJI_SP']
            data['onmyoji']['ONMYOJI_SSR'] = config_default_data['onmyoji']['ONMYOJI_SSR']
            data['onmyoji']['ONMYOJI_SR'] = config_default_data['onmyoji']['ONMYOJI_SR']
            data['onmyoji']['ONMYOJI_R'] = config_default_data['onmyoji']['ONMYOJI_R']

        json.dump(data, open(_draw_config, 'w', encoding='utf8'), indent=4, ensure_ascii=False)








import aiohttp
from .config import DRAW_PATH, SEMAPHORE
from asyncio.exceptions import TimeoutError
from bs4 import BeautifulSoup
from .util import download_img
from .util import remove_prohibited_str
from urllib.parse import unquote
import bs4
import asyncio

try:
    import ujson as json
except ModuleNotFoundError:
    import json

headers = {'User-Agent': '"Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; TencentTraveler 4.0)"'}


async def update_simple_info(url: str, game_name: str) -> 'dict, int':
    try:
        with open(DRAW_PATH + f'{game_name}.json', 'r', encoding='utf8') as f:
            data = json.load(f)
    except (ValueError, FileNotFoundError):
        data = {}
    try:
        async with aiohttp.ClientSession(headers=headers) as session:
            async with session.get(url, timeout=7) as response:
                soup = BeautifulSoup(await response.text(), 'lxml')
                divs = get_char_divs(soup, game_name)
                for div in divs:
                    type_lst = get_type_lst(div, game_name)
                    index = 0
                    for char_lst in type_lst:
                        contents = get_char_lst_contents(char_lst, game_name)
                        for char in contents:
                            data = await retrieve_char_data(char, game_name, data, index)
                        index += 1
            data = await _last_check(data, game_name, session)
    except TimeoutError:
        print(f'更新 {game_name} 超时...')
        return {}, 999
    with open(DRAW_PATH + f'{game_name}.json', 'w', encoding='utf8') as wf:
        wf.write(json.dumps(data, ensure_ascii=False, indent=4))
    return data, 200


# 获取所有包含需要图片的divs
def get_char_divs(soup: bs4.BeautifulSoup, game_name: str) -> bs4.element.ResultSet:
    if game_name == 'pcr':
        return soup.find_all('div', {'class': 'tabbertab'})
    if game_name == 'azur':
        return soup.find_all('div', {'class': 'resp-tabs'})


# 拿到所有类型
def get_type_lst(div: bs4.element.Tag, game_name: str):
    if game_name in ['pcr', 'azur']:
        return div.find('div', {'class': 'resp-tabs-container'}).find_all('div', {'class': 'resp-tab-content'})


# 获取所有角色div
def get_char_lst_contents(char_lst: bs4.element.Tag, game_name: str):
    contents = []
    # print(len(char_lst.find_all('tr')))
    if game_name == 'pcr':
        contents = char_lst.contents
    if game_name == 'azur':
        contents = char_lst.find('table').find('tbody').contents[-1].find('td').contents
    return [x for x in contents if x != '\n']


# 额外数据
async def _last_check(data: dict, game_name: str, session: aiohttp.ClientSession) -> dict:
    if game_name == 'azur':
        idx = 1
        for url in [
            'https://patchwiki.biligame.com/images/blhx/thumb/1/15/pxho13xsnkyb546tftvh49etzdh74cf.png/60px'
            '-舰娘头像外框普通.png',
            'https://patchwiki.biligame.com/images/blhx/thumb/a/a9/k8t7nx6c8pan5vyr8z21txp45jxeo66.png/60px'
            '-舰娘头像外框稀有.png',
            'https://patchwiki.biligame.com/images/blhx/thumb/a/a5/5whkzvt200zwhhx0h0iz9qo1kldnidj.png/60px'
            '-舰娘头像外框精锐.png',
            'https://patchwiki.biligame.com/images/blhx/thumb/a/a2/ptog1j220x5q02hytpwc8al7f229qk9.png/60px-'
            '舰娘头像外框超稀有.png'
        ]:
            await download_img(url, 'azur', f'{idx}_star')
            idx += 1
        tasks = []
        semaphore = asyncio.Semaphore(SEMAPHORE)
        for key in data.keys():
            tasks.append(asyncio.ensure_future(_async_update_azur_extra_info(key, session, semaphore)))
        asyResult = await asyncio.gather(*tasks)
        for x in asyResult:
            for key in x.keys():
                data[key]['获取途径'] = x[key]['获取途径']
    return data


azur_type = {
    '0': '驱逐',
    '1': '轻巡',
    '2': '重巡',
    '3': '超巡',
    '4': '战巡',
    '5': '战列',
    '6': '航母',
    '7': '航站',
    '8': '轻航',
    '9': '重炮',
    '10': '维修',
    '11': '潜艇',
    '12': '运输',
}


# 整理数据
async def retrieve_char_data(char: bs4.element.Tag, game_name: str, data: dict, index: int = 0) -> dict:
    member_dict = {}
    if game_name == 'pcr':
        member_dict = {
            '头像': unquote(char.find('img', {'class': 'img-kk'})['src']),
            '名称': remove_prohibited_str(char.find('a')['title']),
            '星级': 3 - index}
    if game_name == 'azur':
        char = char.find('td').find('div')
        avatar_img = char.find('a').find('img')
        try:
            member_dict['头像'] = unquote(str(avatar_img['srcset']).split(' ')[-2])
        except KeyError:
            member_dict['头像'] = unquote(str(avatar_img['src']).split(' ')[-2])
        member_dict['名称'] = remove_prohibited_str(str(avatar_img['alt'])[: str(avatar_img['alt']).find('头像')])
        star = char.find('div').find('img')['alt']
        if star == '舰娘头像外框普通.png':
            star = 1
        elif star == '舰娘头像外框稀有.png':
            star = 2
        elif star == '舰娘头像外框精锐.png':
            star = 3
        elif star == '舰娘头像外框超稀有.png':
            star = 4
        elif star == '舰娘头像外框海上传奇.png':
            star = 5
        elif star in ['舰娘头像外框最高方案.png', '舰娘头像外框决战方案.png', '舰娘头像外框超稀有META.png']:
            star = 6
        member_dict['星级'] = star
        member_dict['类型'] = azur_type[str(index)]
    await download_img(member_dict['头像'], game_name, member_dict['名称'])
    data[member_dict['名称']] = member_dict
    print(f'{member_dict["名称"]} is update...')
    return data


async def _async_update_azur_extra_info(key: str, session: aiohttp.ClientSession, semaphore):
    if key[-1] == '改':
        return {key: {'获取途径': ['无法建造']}}
    async with semaphore:
        for i in range(20):
            try:
                async with session.get(f'https://wiki.biligame.com/blhx/{key}', timeout=7) as res:
                    soup = BeautifulSoup(await res.text(), 'lxml')
                    construction_time = str(soup.find('table', {'class': 'wikitable sv-general'}).find('tbody'))
                    x = {key: {'获取途径': []}}
                    if construction_time.find('无法建造') != -1:
                        x[key]['获取途径'].append('无法建造')
                    elif construction_time.find('活动已关闭') != -1:
                        x[key]['获取途径'].append('活动限定')
                    else:
                        x[key]['获取途径'].append('可以建造')
                    print(f'碧蓝航线获取额外信息 {key}...{x[key]["获取途径"]}')
                    return x
            except TimeoutError:
                print(f'访问 https://wiki.biligame.com/blhx/{key} 第 {i}次 超时...已再次访问')
    return {}





from nonebot.adapters.cqhttp import MessageSegment
import nonebot
import random
from .config import PRTS_FIVE_P, PRTS_FOUR_P, PRTS_SIX_P, PRTS_THREE_P, DRAW_PATH, PRTS_FLAG
from .update_game_info import update_info
from .util import generate_img, init_star_rst, max_card, BaseData, UpEvent, set_list, get_star
from .init_card_pool import init_game_pool
from pathlib import Path
from .announcement import PrtsAnnouncement
from dataclasses import dataclass
try:
    import ujson as json
except ModuleNotFoundError:
    import json

driver: nonebot.Driver = nonebot.get_driver()

up_char_file = Path() / "data" / "draw_card" / "draw_card_up" / "prts_up_char.json"

prts_dict = {}
UP_OPERATOR = []
ALL_OPERATOR = []
_CURRENT_POOL_TITLE = ''
POOL_IMG = ''


@dataclass
class Operator(BaseData):
    recruit_only: bool  # 公招限定
    event_only: bool  # 活动获得干员
    # special_only: bool  # 升变/异格干员


async def prts_draw(count: int = 300):
    cnlist = ['★★★★★★', '★★★★★', '★★★★', '★★★']
    star_list = [0, 0, 0, 0]
    operator_list, operator_dict, six_list, star_list, six_index_list = format_card_information(count, star_list)
    up_list = []
    if _CURRENT_POOL_TITLE:
        for x in UP_OPERATOR:
            for operator in x.operators:
                up_list.append(operator)
    rst = init_star_rst(star_list, cnlist, six_list, six_index_list, up_list)
    if count > 90:
        operator_list = set_list(operator_list)
    pool_info = f"当前up池: {_CURRENT_POOL_TITLE}" if _CURRENT_POOL_TITLE else ""
    return pool_info + MessageSegment.image(
        "base64://" + await generate_img(operator_list, 'prts', star_list)) \
           + '\n' + rst[:-1] + '\n' + max_card(operator_dict)


async def update_prts_info():
    global prts_dict, ALL_OPERATOR
    url = 'https://wiki.biligame.com/arknights/干员数据表'
    data, code = await update_info(url, 'prts', ['头像', '名称', '阵营', '星级', '性别', '是否感染', '获取途径', '初始生命', '初始防御',
                                                 '初始法抗', '再部署', '部署费用', '阻挡数', '攻击速度', '标签'])
    if code == 200:
        prts_dict = data
        ALL_OPERATOR = init_game_pool('prts', prts_dict, Operator)
        await _init_up_char()


async def init_prts_data():
    global prts_dict, ALL_OPERATOR
    if PRTS_FLAG:
        with open(DRAW_PATH + 'prts.json', 'r', encoding='utf8') as f:
            prts_dict = json.load(f)
        ALL_OPERATOR = init_game_pool('prts', prts_dict, Operator)
        await _init_up_char()


# 抽取干员
def _get_operator_card(add: float):
    star = get_star([6, 5, 4, 3], [PRTS_SIX_P + add, PRTS_FIVE_P, PRTS_FOUR_P, PRTS_THREE_P])
    if _CURRENT_POOL_TITLE:
        zooms = [x.zoom for x in UP_OPERATOR if x.star == star]
        zoom = 0
        weight = 0
        # 分配概率和权重
        for z in zooms:
            if z < 1:
                zoom = z
            else:
                weight = z
        # UP
        if random.random() < zoom:
            up_operators = [x.operators for x in UP_OPERATOR if x.star == star and x.zoom < 1][0]
            up_operator_name = random.choice(up_operators)
            # print(up_operator_name)
            acquire_operator = [x for x in ALL_OPERATOR if x.name == up_operator_name][0]
        else:
            all_star_operators = [x for x in ALL_OPERATOR if x.star == star
                                  and not any([x.limited, x.event_only, x.recruit_only])]
            weight_up_operators = [x.operators for x in UP_OPERATOR if x.star == star and x.zoom > 1]
            # 权重
            if weight_up_operators and random.random() < 1.0 / float(len(all_star_operators)) * weight:
                up_operator_name = random.choice(weight_up_operators[0])
                acquire_operator = [x for x in ALL_OPERATOR if x.name == up_operator_name][0]
            else:
                acquire_operator = random.choice(all_star_operators)
    else:
        acquire_operator = random.choice([x for x in ALL_OPERATOR if x.star == star
                                          and not any([x.limited, x.event_only, x.recruit_only])])
    return acquire_operator, 6 - star


# 整理数据
def format_card_information(count: int, star_list: list):
    max_star_lst = []       # 获取的最高星级角色列表
    max_index_lst = []      # 获取最高星级角色的次数
    obj_list = []           # 获取所有角色
    obj_dict = {}           # 获取角色次数字典
    add = 0.0
    count_idx = 0
    for i in range(count):
        count_idx += 1
        obj, code = _get_operator_card(add)
        star_list[code] += 1
        if code == 0:
            max_star_lst.append(obj.name)
            max_index_lst.append(i)
            add = 0.0
            count_idx = 0
        elif count_idx > 50:
            add += 0.02
        try:
            obj_dict[obj.name] += 1
        except KeyError:
            obj_dict[obj.name] = 1
        obj_list.append(obj)
    return obj_list, obj_dict, max_star_lst, star_list, max_index_lst


# 获取up干员和概率
async def _init_up_char():
    global _CURRENT_POOL_TITLE, POOL_IMG, UP_OPERATOR
    UP_OPERATOR = []
    up_char_dict = await PrtsAnnouncement.update_up_char()
    _CURRENT_POOL_TITLE = up_char_dict['title']
    if _CURRENT_POOL_TITLE:
        POOL_IMG = MessageSegment.image(up_char_dict['pool_img'])
    up_char_dict = up_char_dict['up_char']
    print(f'成功获取明日方舟当前up信息...当前up池: {_CURRENT_POOL_TITLE}')
    average_dict = {'6': {}, '5': {}, '4': {}}
    for star in up_char_dict.keys():
        for key in up_char_dict[star].keys():
            if average_dict[star].get(up_char_dict[star][key]):
                average_dict[star][up_char_dict[star][key]].append(key)
            else:
                average_dict[star][up_char_dict[star][key]] = [key]
    for star in average_dict.keys():
        for str_zoom in average_dict[star].keys():
            if str_zoom[0] == '权':
                zoom = float(str_zoom[1:])
            else:
                zoom = float(str_zoom) / 100
            UP_OPERATOR.append(UpEvent(star=int(star), operators=average_dict[star][str_zoom], zoom=zoom))


async def reload_prts_pool():
    await _init_up_char()
    return f'当前UP池：{_CURRENT_POOL_TITLE} {POOL_IMG}'




from nonebot.adapters.cqhttp import MessageSegment
import random
from .update_game_requests_info import update_requests_info
from .util import generate_img, init_star_rst, BaseData, set_list, get_star, max_card
from .config import FGO_CARD_FOUR_P, FGO_CARD_FIVE_P, FGO_CARD_THREE_P, FGO_SERVANT_THREE_P, \
    FGO_SERVANT_FIVE_P, FGO_SERVANT_FOUR_P, FGO_FLAG, DRAW_PATH
from dataclasses import dataclass
from .init_card_pool import init_game_pool

try:
    import ujson as json
except ModuleNotFoundError:
    import json

ALL_CHAR = []
ALL_CARD = []


@dataclass
class FgoChar(BaseData):
    pass


async def fgo_draw(count: int):
    #            0      1      2
    cnlist = ['★★★★★', '★★★★', '★★★']
    obj_list, obj_dict, max_star_list, star_list, max_star_index_list = _format_card_information(count)
    rst = init_star_rst(star_list, cnlist, max_star_list, max_star_index_list)
    if count > 90:
        obj_list = set_list(obj_list)
    return MessageSegment.image("base64://" + await generate_img(obj_list, 'fgo', star_list)) \
           + '\n' + rst[:-1] + '\n' + max_card(obj_dict)


async def update_fgo_info():
    global ALL_CHAR, ALL_CARD
    data, code = await update_requests_info('fgo')
    if code == 200:
        ALL_CHAR = init_game_pool('fgo', data, FgoChar)
    data, code = await update_requests_info('fgo_card')
    if code == 200:
        ALL_CARD = init_game_pool('fgo_card', data, FgoChar)


async def init_fgo_data():
    global ALL_CHAR, ALL_CARD
    if FGO_FLAG:
        with open(DRAW_PATH + 'fgo.json', 'r', encoding='utf8') as f:
            fgo_dict = json.load(f)
        ALL_CHAR = init_game_pool('fgo', fgo_dict, FgoChar)
        with open(DRAW_PATH + 'fgo_card.json', 'r', encoding='utf8') as f:
            fgo_dict = json.load(f)
        ALL_CARD = init_game_pool('fgo', fgo_dict, FgoChar)


# 抽取卡池
def _get_fgo_card(mode: int = 1):
    global ALL_CHAR, ALL_CARD
    if mode == 1:
        star = get_star([8, 7, 6, 5, 4, 3], [FGO_SERVANT_FIVE_P, FGO_SERVANT_FOUR_P, FGO_SERVANT_THREE_P,
                                             FGO_CARD_FIVE_P, FGO_CARD_FOUR_P, FGO_CARD_THREE_P])
    elif mode == 2:
        star = get_star([5, 4], [FGO_CARD_FIVE_P, FGO_CARD_FOUR_P])
    else:
        star = get_star([8, 7, 6], [FGO_SERVANT_FIVE_P, FGO_SERVANT_FOUR_P, FGO_SERVANT_THREE_P])
    if star > 5:
        itype = 'servant'
        star -= 3
        chars = [x for x in ALL_CHAR if x.star == star if not x.limited]
    else:
        itype = 'card'
        chars = [x for x in ALL_CARD if x.star == star if not x.limited]
    return random.choice(chars), 5 - star, itype


# 整理数据
def _format_card_information(count: int):
    max_star_lst = []  # 获取的最高星级角色列表
    max_index_lst = []  # 获取最高星级角色的次数
    star_list = [0, 0, 0]
    obj_list = []  # 获取所有角色
    obj_dict = {}  # 获取角色次数字典
    servant_count = 0  # 保底计算
    card_count = 0  # 保底计算
    for i in range(count):
        servant_count += 1
        card_count += 1
        # 四星卡片保底
        if card_count == 9:
            obj, code, itype = _get_fgo_card(2)
        # 三星从者保底
        elif servant_count == 10:
            obj, code, itype = _get_fgo_card(3)
            _count = 0
        # 普通抽
        else:
            obj, code, itype = _get_fgo_card()
        star_list[code] += 1
        if itype == 'card' and code < 2:
            card_count = 0
        if itype == 'servant':
            servant_count = 0
        if code == 0:
            max_star_lst.append(obj.name)
            max_index_lst.append(i)
        try:
            obj_dict[obj.name] += 1
        except KeyError:
            obj_dict[obj.name] = 1
        obj_list.append(obj)
    return obj_list, obj_dict, max_star_lst, star_list, max_index_lst



#coding:utf-8
import aiohttp
from .config import DRAW_PATH
from asyncio.exceptions import TimeoutError
from bs4 import BeautifulSoup
from .util import download_img
from urllib.parse import unquote
from .util import remove_prohibited_str
import bs4
import re
try:
    import ujson as json
except ModuleNotFoundError:
    import json


headers = {'User-Agent': '"Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; TencentTraveler 4.0)"'}


async def update_info(url: str, game_name: str, info_list: list = None) -> 'dict, int':
    try:
        with open(DRAW_PATH + f'{game_name}.json', 'r', encoding='utf8') as f:
            data = json.load(f)
    except (ValueError, FileNotFoundError):
        data = {}
    try:
        async with aiohttp.ClientSession(headers=headers) as session:
            async with session.get(url, timeout=7) as response:
                soup = BeautifulSoup(await response.text(), 'lxml')
                _tbody = get_tbody(soup, game_name, url)
                trs = _tbody.find_all('tr')
                att_dict, start_index, index = init_attr(game_name)
                if game_name == 'guardian':
                    start_index = 1
                if game_name == 'azur':
                    start_index = 0
                for th in trs[0].find_all('th')[start_index:]:
                    text = th.text
                    if text[-1] == '\n':
                        text = text[:-1]
                    att_dict[text] = index
                    index += 1
                for tr in trs[1:]:
                    member_dict = {}
                    tds = tr.find_all('td')
                    if not info_list:
                        info_list = att_dict.keys()
                    for key in info_list:
                        key, attr = parse_key(key, game_name)
                        td = tds[att_dict[key]]
                        last_tag = unquote(_find_last_tag(td, attr, game_name), 'utf-8')
                        member_dict[key] = last_tag
                        member_dict = intermediate_check(member_dict, key, game_name, td)
                    avatar_img = await _modify_avatar_url(session, game_name, member_dict["名称"])
                    member_dict['头像'] = avatar_img if avatar_img else member_dict['头像']
                    member_dict, name = replace_update_name(member_dict, game_name)
                    await download_img(member_dict['头像'], game_name, name)
                    data[name] = member_dict
                    print(f'{name} is update...')
            data = await _last_check(data, game_name, session)
    except TimeoutError:
        print(f'更新 {game_name} 超时...')
        return {}, 999
    with open(DRAW_PATH + f'{game_name}.json', 'w', encoding='utf8') as wf:
        wf.write(json.dumps(data, ensure_ascii=False, indent=4))
    return data, 200


def _find_last_tag(element: bs4.element.Tag, attr: str, game_name: str) -> str:
    last_tag = []
    for des in element.descendants:
        last_tag.append(des)
    if len(last_tag) == 1 and last_tag[0] == '\n':
        last_tag = ''
    elif last_tag[-1] == '\n':
        last_tag = last_tag[-2]
    else:
        last_tag = last_tag[-1]
    if attr and str(last_tag):
        last_tag = last_tag[attr]
    elif str(last_tag).find('<img') != -1:
        if last_tag.get('srcset'):
            last_tag = str(last_tag.get('srcset')).strip().split(' ')[-2].strip()
        else:
            last_tag = last_tag['src']
    else:
        last_tag = str(last_tag)
    if str(last_tag) and str(last_tag)[-1] == '\n':
        last_tag = str(last_tag)[:-1]

    if game_name not in ['pretty', 'pretty_card', 'guardian'] and last_tag.find('http') == -1:
        last_tag = last_tag.split('.')[0]

    return last_tag


# 获取大图（小图快爬）
async def _modify_avatar_url(session: aiohttp.ClientSession, game_name: str, char_name: str):
    if game_name == 'prts':
        async with session.get(f'https://wiki.biligame.com/arknights/{char_name}', timeout=7) as res:
            soup = BeautifulSoup(await res.text(), 'lxml')
            try:
                img_url = str(soup.find('img', {'class': 'img-bg'})['srcset']).split(' ')[-2]
            except KeyError:
                img_url = str(soup.find('img', {'class': 'img-bg'})['src'])
            return img_url
    if game_name == 'genshin':
        return None
    if game_name == 'pretty_card':
        async with session.get(f'https://wiki.biligame.com/umamusume/{char_name}', timeout=7) as res:
            soup = BeautifulSoup(await res.text(), 'lxml')
            img_url = soup.find('div', {'class': 'support_card-left'}).find('div').find('img').get('src')
            return img_url
    if game_name == 'guardian':
        # 未上传图片太多，换成像素图
        # async with session.get(f'https://wiki.biligame.com/gt/{char_name}', timeout=7) as res:
        #     soup = BeautifulSoup(await res.text(), 'lxml')
        #     soup = soup.find('table', {'class': 'wikitable'}).find('tbody').find('tr')
        #     try:
        #         img_url = str(soup.find('img', {'class': 'img-kk'})['srcset']).split(' ')[-2]
        #     except KeyError:
        #         img_url = str(soup.find('img', {'class': 'img-kk'})['src'])
        #     except TypeError:
        #         print(f'{char_name} 图片还未上传，跳过...')
        #         img_url = ''
        #     return img_url
        return None


# 数据最后处理（是否需要额外数据或处理数据）
async def _last_check(data: dict, game_name: str, session: aiohttp.ClientSession):
    # if game_name == 'prts':
    #     tasks = []
    #     for key in data.keys():
    #         tasks.append(asyncio.ensure_future(_async_update_prts_extra_info(key, session)))
    #     asyResult = await asyncio.gather(*tasks)
    #     for x in asyResult:
    #         for key in x.keys():
    #             data[key]['获取途径'] = x[key]['获取途径']
    if game_name == 'genshin':
        for key in data.keys():
            async with session.get(f'https://wiki.biligame.com/ys/{key}', timeout=7) as res:
                soup = BeautifulSoup(await res.text(), 'lxml')
                _trs = ''
                for table in soup.find_all('table', {'class': 'wikitable'}):
                    if str(table).find('常驻/限定') != -1:
                        _trs = table.find('tbody').find_all('tr')
                        break
                for tr in _trs:
                    data[key]['常驻/限定'] = '未知'
                    if str(tr).find('限定UP') != -1:
                        data[key]['常驻/限定'] = '限定UP'
                        print(f'原神获取额外数据 {key}...{data[key]["常驻/限定"]}')
                        break
                    elif str(tr).find('常驻UP') != -1:
                        data[key]['常驻/限定'] = '常驻UP'
                        print(f'原神获取额外数据 {key}...{data[key]["常驻/限定"]}')
                        break
    if game_name == 'pretty':
        for keys in data.keys():
            for key in data[keys].keys():
                r = re.search(r'.*?40px-(.*)图标.png', str(data[keys][key]))
                if r:
                    data[keys][key] = r.group(1)
                    print(f'赛马娘额外修改数据...{keys}[{key}]=> {r.group(1)}')
    if game_name == 'guardian':
        for keys in data.keys():
            for key in data[keys].keys():
                r = re.search(r'.*?-star_(.*).png', str(data[keys][key]))
                if r:
                    data[keys][key] = r.group(1)
                    print(f'坎公骑士剑额外修改数据...{keys}[{key}] => {r.group(1)}')
    return data


# 对抓取每行数据是否需要额外处理？
def intermediate_check(member_dict: dict, key: str, game_name: str, td: bs4.element.Tag):
    if game_name == 'prts':
        if key == '获取途径':
            msg = re.search('<td.*?>([\\s\\S]*)</td>', str(td)).group(1).strip()
            msg = msg[:-1] if msg[-1] == '\n' else msg
            if msg.find('<a') != -1:
                for a in td.find_all('a'):
                    msg = msg.replace(str(a), a.text)
            member_dict['获取途径'] = msg.split('<br/>')
    if game_name == 'pretty':
        if key == '初始星级':
            member_dict['初始星级'] = len(td.find_all('img'))
    if game_name == 'pretty_card':
        if key == '获取方式':
            obtain = []
            for x in str(td.text).replace('\n', '').strip().split('、'):
                if x:
                    obtain.append(x)
            member_dict['获取方式'] = obtain
    if game_name == 'guardian':
        if key == '头像':
            member_dict['星级'] = str(td.find('span').find('img')['alt'])[-5]
            try:
                member_dict['头像'] = str(td.find('img')['srcset']).split(' ')[0]
            except KeyError:
                member_dict['头像'] = str(td.find('img')['src'])
    return member_dict


def init_attr(game_name: str):
    att_dict = {'头像': 0, '名称': 1}
    start_index = 2
    index = 2
    if game_name == 'guardian':
        att_dict = {'头像': 0, '名称': 0}
        start_index = 1
        index = 1
    return att_dict, start_index, index


# 解析key
def parse_key(key: str, game_name):
    attr = ''
    if game_name == 'genshin_arms':
        if key.find('.') != -1:
            key = key.split('.')
            attr = key[-1]
            key = key[0]
    return key, attr


# 拿到名称
def replace_update_name(member_dict: dict, game_name: str):
    name = member_dict['名称']
    if game_name == 'pretty_card':
        name = member_dict['中文名']
        name = remove_prohibited_str(name)
        member_dict['中文名'] = name
    else:
        name = remove_prohibited_str(name)
        member_dict['名称'] = name
    return member_dict, name


# 拿到tbody，不同游戏tbody可能不同
def get_tbody(soup: bs4.BeautifulSoup, game_name: str, url: str):
    max_count = 0
    _tbody = None
    if game_name == 'guardian_arms':
        if url[-2:] == '盾牌':
            div = soup.find('div', {'class': 'resp-tabs-container'}).find_all('div', {'class': 'resp-tab-content'})[1]
            _tbody = div.find('tbody')
        else:
            div = soup.find('div', {'class': 'resp-tabs-container'}).find_all('div', {'class': 'resp-tab-content'})[0]
            _tbody = div.find('table', {'id': 'CardSelectTr'}).find('tbody')
    else:
        for tbody in soup.find_all('tbody'):
            if len(tbody.find_all('tr')) > max_count:
                _tbody = tbody
                max_count = len(tbody.find_all('tr'))
    return _tbody


# async def _async_update_prts_extra_info(key: str, session: aiohttp.ClientSession):
#     for i in range(10):
#         try:
#             async with session.get(f'https://wiki.biligame.com/arknights/{key}', timeout=7) as res:
#                 soup = BeautifulSoup(await res.text(), 'lxml')
#                 obtain = str(soup.find('table', {'class': 'wikitable'}).find('tbody').find_all('td')[-1])
#                 obtain = re.search(r'<td.*?>([\s\S]*)</.*?', obtain).group(1)
#                 obtain = obtain[:-1] if obtain[-1] == '\n' else obtain
#                 if obtain.find('<br/>'):
#                     obtain = obtain.split('<br/>')
#                 elif obtain.find('<br>'):
#                     obtain = obtain.split('<br>')
#                 for i in range(len(obtain)):
#                     if obtain[i].find('<a') != -1:
#                         text = ''
#                         for msg in obtain[i].split('</a>'):
#                             r = re.search('>(.*)', msg)
#                             if r:
#                                 text += r.group(1) + ' '
#                         obtain[i] = obtain[i].split('<a')[0] + text[:-1] + obtain[i].split('</a>')[-1]
#                 print(f'明日方舟获取额外信息 {key}...{obtain}')
#                 x = {key: {}}
#                 x[key]['获取途径'] = obtain
#                 return x
#         except TimeoutError:
#             print(f'访问 https://wiki.biligame.com/arknights/{key} 第 {i}次 超时...已再次访问')
#     return {}




import aiohttp
from .config import DRAW_PATH, SEMAPHORE
from asyncio.exceptions import TimeoutError
from .util import download_img
from bs4 import BeautifulSoup
from .util import remove_prohibited_str
import asyncio
try:
    import ujson as json
except ModuleNotFoundError:
    import json


headers = {'User-Agent': '"Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; TencentTraveler 4.0)"'}


async def update_requests_info(game_name: str):
    try:
        with open(DRAW_PATH + f'{game_name}.json', 'r', encoding='utf8') as f:
            data = json.load(f)
    except (ValueError, FileNotFoundError):
        data = {}
    try:
        async with aiohttp.ClientSession(headers=headers) as session:
            if game_name in ['fgo', 'fgo_card']:
                if game_name == 'fgo':
                    url = 'http://fgo.vgtime.com/servant/ajax?card=&wd=&ids=&sort=12777&o=desc&pn='
                else:
                    url = 'http://fgo.vgtime.com/equipment/ajax?wd=&ids=&sort=12958&o=desc&pn='
                for i in range(9999):
                    async with session.get(f'{url}{i}', timeout=7) as response:
                        fgo_data = json.loads(await response.text())
                        if int(fgo_data['nums']) == 0:
                            break
                        for x in fgo_data['data']:
                            x['name'] = remove_prohibited_str(x['name'])
                            key = x['name']
                            data = add_to_data(data, x, game_name)
                            await download_img(data[key]['头像'], game_name, key)
                            print(f'{key} is update...')
            if game_name == 'onmyoji':
                url = 'https://yys.res.netease.com/pc/zt/20161108171335/js/app/all_shishen.json?v74='
                async with session.get(f'{url}', timeout=7) as response:
                    onmyoji_data = await response.json()
                    for x in onmyoji_data:
                        x['name'] = remove_prohibited_str(x['name'])
                        key = x['name']
                        data = add_to_data(data, x, game_name)
                        print(f'{key} is update...')
            data = await _last_check(data, game_name, session)
    except TimeoutError:
        print(f'更新 {game_name} 超时...')
        return {}, 999
    with open(DRAW_PATH + f'{game_name}.json', 'w', encoding='utf8') as wf:
        json.dump(data, wf, ensure_ascii=False, indent=4)
    return data, 200


# 添加到字典
def add_to_data(data: dict, x: dict, game_name: str) -> dict:
    member_dict = {}
    if game_name == 'fgo':
        member_dict = {
            'id': x['id'],
            'card_id': x['charid'],
            '头像': x['icon'],
            '名称': x['name'],
            '职阶': x['classes'],
            '星级': x['star'],
            'hp': x['lvmax4hp'],
            'atk': x['lvmax4atk'],
            'card_quick': x['cardquick'],
            'card_arts': x['cardarts'],
            'card_buster': x['cardbuster'],
            '宝具': x['tprop'],
        }
    if game_name == 'fgo_card':
        member_dict = {
            'id': x['id'],
            'card_id': x['equipid'],
            '头像': x['icon'],
            '名称': x['name'],
            '星级': x['star'],
            'hp': x['lvmax_hp'],
            'atk': x['lvmax_atk'],
            'skill_e': x['skill_e'].split('<br />')[: -1],
        }
    if game_name == 'onmyoji':
        member_dict = {
            'id': x['id'],
            '名称': x['name'],
            '星级': x['level'],
        }
    data[member_dict['名称']] = member_dict
    return data


# 获取额外数据
async def _last_check(data: dict, game_name: str, session: aiohttp.ClientSession) -> dict:
    if game_name == 'fgo':
        url = 'http://fgo.vgtime.com/servant/'
        tasks = []
        semaphore = asyncio.Semaphore(SEMAPHORE)
        for key in data.keys():
            tasks.append(asyncio.ensure_future(
                _async_update_fgo_extra_info(url, key, data[key]['id'], session, semaphore)))
        asyResult = await asyncio.gather(*tasks)
        for x in asyResult:
            for key in x.keys():
                data[key]['入手方式'] = x[key]['入手方式']
    if game_name == 'onmyoji':
        url = 'https://yys.163.com/shishen/{}.html'
        for key in data.keys():
            async with session.get(f'{url.format(data[key]["id"])}', timeout=7) as response:
                soup = BeautifulSoup(await response.text(), 'lxml')
                data[key]['头像'] = "https:" + soup.find('div', {'class': 'pic_wrap'}).find('img')['src']
                await download_img(data[key]['头像'], game_name, key)
    return data


async def _async_update_fgo_extra_info(url: str, key: str, _id: str, session: aiohttp.ClientSession, semaphore):
    # 防止访问超时
    async with semaphore:
        for i in range(10):
            try:
                async with session.get(f'{url}{_id}', timeout=7) as response:
                    soup = BeautifulSoup(await response.text(), 'lxml')
                    obtain = soup.find('table', {'class': 'uk-table uk-codex-table'}).find_all('td')[-1].text
                    if obtain.find('限时活动免费获取 活动结束后无法获得') != -1:
                        obtain = ['活动获取']
                    elif obtain.find('非限时UP无法获得') != -1:
                        obtain = ['限时召唤']
                    else:
                        if obtain.find('&') != -1:
                            obtain = obtain.strip().split('&')
                        else:
                            obtain = obtain.strip().split(' ')
                    print(f'Fgo获取额外信息 {key}....{obtain}')
                    x = {key: {}}
                    x[key]['入手方式'] = obtain
                    return x
            except TimeoutError:
                print(f'访问{url}{_id} 第 {i}次 超时...已再次访问')
    return {}









from typing import Any


def init_game_pool(game: str, data: dict, Operator: Any):
    tmp_lst = []
    if game == 'prts':
        for key in data.keys():
            limited = False
            recruit_only = False
            event_only = False
            if '限定寻访' in data[key]['获取途径']:
                limited = True
            if '干员寻访' not in data[key]['获取途径'] and '公开招募' in data[key]['获取途径']:
                recruit_only = True
            if '活动获取' in data[key]['获取途径']:
                event_only = True
            if '干员寻访' not in data[key]['获取途径'] and '凭证交易所' == data[key]['获取途径'][0]:
                limited = True
            if '干员寻访' not in data[key]['获取途径'] and '信用累计奖励' == data[key]['获取途径'][0]:
                limited = True
            if key.find('阿米娅') != -1:
                continue
            tmp_lst.append(Operator(name=key, star=int(data[key]['星级']),
                                    limited=limited, recruit_only=recruit_only, event_only=event_only))
    if game == 'genshin':
        for key in data.keys():
            if key.find('旅行者') != -1:
                continue
            limited = False
            if data[key]['常驻/限定'] == '限定UP':
                limited = True
            tmp_lst.append(Operator(name=key, star=int(data[key]['稀有度'][:1]), limited=limited))
    if game == 'genshin_arms':
        for key in data.keys():
            if data[key]['获取途径'].find('祈愿') != -1:
                limited = False
                if data[key]['获取途径'].find('限定祈愿') != -1:
                    limited = True
                tmp_lst.append(Operator(name=key, star=int(data[key]['稀有度'][:1]), limited=limited))
    if game == 'pretty':
        for key in data.keys():
            tmp_lst.append(Operator(name=key, star=data[key]['初始星级'], limited=False))
    if game == 'pretty_card':
        for key in data.keys():
            limited = False
            if '卡池' not in data[key]['获取方式']:
                limited = True
            if not data[key]['获取方式']:
                limited = False
            tmp_lst.append(Operator(name=data[key]['中文名'], star=len(data[key]['稀有度']), limited=limited))
    if game in ['guardian', 'guardian_arms']:
        for key in data.keys():
            tmp_lst.append(Operator(name=data[key]['名称'], star=int(data[key]['星级']), limited=False))
    if game == 'pcr':
        for key in data.keys():
            limited = False
            if key.find('（') != -1:
                limited = True
            tmp_lst.append(Operator(name=data[key]['名称'], star=int(data[key]['星级']), limited=limited))
    if game == 'azur':
        for key in data.keys():
            limited = False
            if '可以建造' not in data[key]['获取途径']:
                limited = True
            tmp_lst.append(Operator(name=data[key]['名称'], star=int(data[key]['星级']),
                                    limited=limited, itype=data[key]['类型']))
    if game in ['fgo', 'fgo_card']:
        for key in data.keys():
            limited = False
            try:
                if "圣晶石召唤" not in data[key]['入手方式'] and "圣晶石召唤（Story卡池）" not in data[key]['入手方式']:
                    limited = True
            except KeyError:
                pass
            tmp_lst.append(Operator(name=data[key]['名称'], star=int(data[key]['星级']), limited=limited))
    if game == 'onmyoji':
        for key in data.keys():
            limited = False
            if key in ['奴良陆生', '卖药郎', '鬼灯', '阿香', '蜜桃&芥子', '犬夜叉', '杀生丸', '桔梗', '朽木露琪亚', '黑崎一护',
                       '灶门祢豆子', '灶门炭治郎']:
                limited = True
            tmp_lst.append(Operator(name=data[key]['名称'], star=data[key]['星级'], limited=limited))
    # print(tmp_lst)
    return tmp_lst




import aiohttp
from bs4 import BeautifulSoup
import re
from datetime import datetime
from .config import DRAW_PATH
from pathlib import Path
from asyncio.exceptions import TimeoutError

try:
    import ujson as json
except ModuleNotFoundError:
    import json

headers = {'User-Agent': '"Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; TencentTraveler 4.0)"'}

prts_up_char = Path(DRAW_PATH + "/draw_card_up/prts_up_char.json")
genshin_up_char = Path(DRAW_PATH + "/draw_card_up/genshin_up_char.json")
pretty_up_char = Path(DRAW_PATH + "/draw_card_up/pretty_up_char.json")
guardian_up_char = Path(DRAW_PATH + "/draw_card_up/guardian_up_char.json")

prts_url = "https://wiki.biligame.com/arknights/%E6%96%B0%E9%97%BB%E5%85%AC%E5%91%8A"
genshin_url = "https://wiki.biligame.com/ys/%E7%A5%88%E6%84%BF"
pretty_url = "https://wiki.biligame.com/umamusume/%E5%85%AC%E5%91%8A"
guardian_url = "https://wiki.biligame.com/gt/%E9%A6%96%E9%A1%B5"


# 是否过时
def is_expired(data: dict):
    times = data['time'].split('-')
    for i in range(len(times)):
        times[i] = str(datetime.now().year) + '-' + times[i].split('日')[0].strip().replace('月', '-')
    start_date = datetime.strptime(times[0], '%Y-%m-%d').date()
    end_date = datetime.strptime(times[1], '%Y-%m-%d').date()
    now = datetime.now().date()
    return start_date <= now <= end_date


# 检查写入
def check_write(data: dict, up_char_file, game_name: str = ''):
    tmp = data
    if game_name in ['genshin', 'pretty', 'guardian']:
        tmp = data['char']
    if not is_expired(tmp):
        if game_name in ['genshin', 'guardian']:
            data['char']['title'] = ''
            data['arms']['title'] = ''
        elif game_name in ['pretty']:
            data['char']['title'] = ''
            data['card']['title'] = ''
        else:
            data['title'] = ''
    else:
        with open(up_char_file, 'w', encoding='utf8') as f:
            json.dump(data, f, indent=4, ensure_ascii=False)
    if not up_char_file.exists():
        with open(up_char_file, 'w', encoding='utf8') as f:
            json.dump(data, f, indent=4, ensure_ascii=False)
    else:
        with open(up_char_file, 'r', encoding='utf8') as f:
            old_data = json.load(f)
            tmp = old_data
            if game_name in ['genshin', 'pretty', 'guardian']:
                tmp = old_data['char']
        if is_expired(tmp):
            return old_data
        else:
            with open(up_char_file, 'w', encoding='utf8') as f:
                json.dump(data, f, indent=4, ensure_ascii=False)
    return data


def _get_up_char(r: str, text: str):
    pr = re.search(r, text)
    chars = pr.group(1)
    probability = pr.group(2)
    chars = chars.replace('[限定]', '').replace('[', '').replace(']', '')
    probability = probability.replace('【', '')
    return chars, probability


class PrtsAnnouncement:

    @staticmethod
    async def get_announcement_text():
        async with aiohttp.ClientSession(headers=headers) as session:
            async with session.get(prts_url, timeout=7) as res:
                soup = BeautifulSoup(await res.text(), 'lxml')
                trs = soup.find('table').find('tbody').find_all('tr')
                for tr in trs:
                    a = tr.find_all('td')[-1].find('a')
                    if a.text.find('寻访') != -1:
                        url = a.get('href')
                        break
            async with session.get(f'https://wiki.biligame.com/{url}', timeout=7) as res:
                return await res.text(), a.text[:-4]

    @staticmethod
    async def update_up_char():
        prts_up_char.parent.mkdir(parents=True, exist_ok=True)
        data = {'up_char': {'6': {}, '5': {}, '4': {}}, 'title': '', 'time': '', 'pool_img': ''}
        try:
            text, title = await PrtsAnnouncement.get_announcement_text()
            soup = BeautifulSoup(text, 'lxml')
            data['title'] = title
            context = soup.find('div', {'id': 'mw-content-text'}).find('div')
            data['pool_img'] = str(context.find('div', {'class': 'center'}).find('div').find('a').
                                   find('img').get('srcset')).split(' ')[-2]
            # print(context.find_all('p'))
            for p in context.find_all('p')[1:]:
                if p.text.find('活动时间') != -1:
                    pr = re.search(r'.*?活动时间：(.*)', p.text)
                    data['time'] = pr.group(1)
                elif p.text.find('★★★★★★') != -1:
                    chars, probability = _get_up_char(r'.*?★★★★★★：(.*?)（.*?出率的?(.*?)%.*?）.*?', p.text)
                    slt = '/'
                    if chars.find('\\') != -1:
                        slt = '\\'
                    for char in chars.split(slt):
                        data['up_char']['6'][char.strip()] = probability.strip()
                elif p.text.find('★★★★★') != -1:
                    chars, probability = _get_up_char(r'.*?★★★★★：(.*?)（.*?出率的?(.*?)%.*?）.*?', p.text)
                    slt = '/'
                    if chars.find('\\') != -1:
                        slt = '\\'
                    for char in chars.split(slt):
                        data['up_char']['5'][char.strip()] = probability.strip()
                elif p.text.find('★★★★') != -1:
                    chars, probability = _get_up_char(r'.*?★★★★：(.*?)（.*?出率的?(.*?)%.*?）.*?', p.text)
                    slt = '/'
                    if chars.find('\\') != -1:
                        slt = '\\'
                    for char in chars.split(slt):
                        data['up_char']['4'][char.strip()] = probability.strip()
                    break
                pr = re.search(r'.*?★：(.*?)（在(.*?)★.*?以(.*?)倍权值.*?）.*?', p.text)
                if pr:
                    char = pr.group(1)
                    star = pr.group(2)
                    weight = pr.group(3)
                    char = char.replace('[限定]', '').replace('[', '').replace(']', '')
                    data['up_char'][star][char.strip()] = f'权{weight}'
            # data['time'] = '03月09日16:00 - 05月23日03:59'
        except TimeoutError:
            print(f'更新明日方舟UP池信息超时...')
            with open(prts_up_char, 'r', encoding='utf8') as f:
                data = json.load(f)
        return check_write(data, prts_up_char)


class GenshinAnnouncement:

    @staticmethod
    async def get_announcement_text():
        async with aiohttp.ClientSession(headers=headers) as session:
            async with session.get(genshin_url, timeout=7) as res:
                return await res.text()

    @staticmethod
    async def update_up_char():
        genshin_up_char.parent.mkdir(exist_ok=True, parents=True)
        data = {
            'char': {'up_char': {'5': {}, '4': {}}, 'title': '', 'time': '', 'pool_img': ''},
            'arms': {'up_char': {'5': {}, '4': {}}, 'title': '', 'time': '', 'pool_img': ''}
        }
        text = await GenshinAnnouncement.get_announcement_text()
        soup = BeautifulSoup(text, 'lxml')
        try:
            div = soup.find_all('div', {'class': 'row'})[1]
            tables = div.find_all('table', {'class': 'wikitable'})
            for table in tables:
                trs = table.find('tbody').find_all('tr')
                pool_img = trs[0].find('th').find('img')
                if pool_img['title'].find('角色活动') == -1:
                    itype = 'arms'
                else:
                    itype = 'char'
                try:
                    data[itype]['pool_img'] = str(pool_img['srcset']).split(' ')[0]
                except KeyError:
                    data[itype]['pool_img'] = pool_img['src']
                data[itype]['title'] = str(pool_img['title']).split(f'期{"角色" if itype == "char" else "武器"}')[0][:-3]
                data[itype]['time'] = trs[1].find('td').text
                if data[itype]['time'][-1] == '\n':
                    data[itype]['time'] = data[itype]['time'][:-1]
                tmp = ''
                for tm in data[itype]['time'].split('~'):
                    date_time_sp = tm.split('/')
                    date_time_sp[2] = date_time_sp[2].strip().replace(' ', '日 ')
                    tmp += date_time_sp[1] + '月' + date_time_sp[2] + ' - '
                data[itype]['time'] = tmp[:-2].strip()
                for a in trs[2].find('td').find_all('a'):
                    char_name = a['title']
                    data[itype]['up_char']['5'][char_name] = "50"
                for a in trs[3].find('td').find_all('a'):
                    char_name = a['title']
                    data[itype]['up_char']['4'][char_name] = "50"
        except TimeoutError:
            print(f'更新原神UP池信息超时...')
            with open(genshin_up_char, 'r', encoding='utf8') as f:
                data = json.load(f)
        except Exception as e:
            print(f'更新原神UP失败，疑似UP池已结束， e：{e}')
            with open(genshin_up_char, 'r', encoding='utf8') as f:
                data = json.load(f)
                data['char']['title'] = ''
                data['arms']['title'] = ''
            with open(genshin_up_char, 'w', encoding='utf8') as wf:
                json.dump(data, wf, ensure_ascii=False, indent=4)
            return data
        return check_write(data, genshin_up_char, 'genshin')


class PrettyAnnouncement:
    @staticmethod
    async def get_announcement_text():
        async with aiohttp.ClientSession(headers=headers) as session:
            async with session.get(pretty_url, timeout=7) as res:
                soup = BeautifulSoup(await res.text(), 'lxml')
                divs = soup.find('div', {'id': 'mw-content-text'}).find('div').find_all('div')
                for div in divs:
                    a = div.find('a')
                    try:
                        title = a['title']
                    except (KeyError, TypeError):
                        continue
                    if title.find('新角色追加') != -1:
                        url = a['href']
                        break
            async with session.get(f'https://wiki.biligame.com/{url}', timeout=7) as res:
                return await res.text(), title[:-2]

    @staticmethod
    async def update_up_char():
        data = {
            'char': {'up_char': {'3': {}, '2': {}, '1': {}}, 'title': '', 'time': '', 'pool_img': ''},
            'card': {'up_char': {'3': {}, '2': {}, '1': {}}, 'title': '', 'time': '', 'pool_img': ''}
        }
        try:
            text, title = await PrettyAnnouncement.get_announcement_text()
            soup = BeautifulSoup(text, 'lxml')
            context = soup.find('div', {'class': 'toc-sticky'})
            if not context:
                context = soup.find('div', {'class': 'mw-parser-output'})
            data['char']['title'] = title
            data['card']['title'] = title
            time = str(context.find_all('big')[1].text)
            time = time.replace('～', '-').replace('/', '月').split(' ')
            time = time[0] + '日 ' + time[1] + ' - ' + time[3] + '日 ' + time[4]
            data['char']['time'] = time
            data['card']['time'] = time
            for p in context.find_all('p'):
                if str(p).find('当期UP赛马娘') != -1:
                    data['char']['pool_img'] = p.find('img')['src']
                    r = re.findall(r'.*?当期UP赛马娘([\s\S]*)＜奖励内容＞.*?', str(p))
                    if r:
                        for x in r:
                            x = str(x).split('\n')
                            for msg in x:
                                if msg.find('★') != -1:
                                    msg = msg.replace('<br/>', '')
                                    msg = msg.split(' ')
                                    if (star := len(msg[0].strip())) == 3:
                                        data['char']['up_char']['3'][msg[1]] = '70'
                                    elif star == 2:
                                        data['char']['up_char']['2'][msg[1]] = '70'
                                    elif star == 1:
                                        data['char']['up_char']['1'][msg[1]] = '70'
                if str(p).find('（当期UP对象）') != -1 and str(p).find('赛马娘') == -1:
                    data['card']['pool_img'] = p.find('img')['src']
                    r = re.search(r'■全?新?支援卡（当期UP对象）([\s\S]*)</p>', str(p))
                    if r:
                        rmsg = r.group(1)
                        rmsg = rmsg.split('<br/>')
                        for x in rmsg[1:]:
                            x = x.replace('\n', '').replace('・', '')
                            x = x.split(' ')
                            if x[0] == 'SSR':
                                data['card']['up_char']['3'][x[1]] = '70'
                            if x[0] == 'SR':
                                data['card']['up_char']['2'][x[1]] = '70'
                            if x[0] == 'R':
                                data['card']['up_char']['1'][x[1]] = '70'
            # 日文->中文
            with open(DRAW_PATH + 'pretty_card.json', 'r', encoding='utf8') as f:
                all_data = json.load(f)
            for star in data['card']['up_char'].keys():
                for name in list(data['card']['up_char'][star].keys()):
                    char_name = name.split(']')[1].strip()
                    tp_name = name[name.find('['): name.find(']') + 1].strip().replace('[', '【').replace(']', '】')
                    for x in all_data.keys():
                        if all_data[x]['名称'].find(tp_name) != -1 and all_data[x]['关联角色'] == char_name:
                            data['card']['up_char'][star].pop(name)
                            data['card']['up_char'][star][all_data[x]['中文名']] = '70'
        except TimeoutError:
            print(f'更新赛马娘UP池信息超时...')
            with open(pretty_up_char, 'r', encoding='utf8') as f:
                data = json.load(f)
        except Exception as e:
            print(f'赛马娘up更新失败 {type(e)}：{e}')
        return check_write(data, pretty_up_char, 'pretty')


class GuardianAnnouncement:

    @staticmethod
    async def get_announcement_text():
        async with aiohttp.ClientSession(headers=headers) as session:
            async with session.get(guardian_url, timeout=7) as res:
                return await res.text()

    @staticmethod
    async def update_up_char():
        data = {
            'char': {'up_char': {'3': {}}, 'title': '', 'time': '', 'pool_img': ''},
            'arms': {'up_char': {'5': {}}, 'title': '', 'time': '', 'pool_img': ''}
        }
        try:
            text = await GuardianAnnouncement.get_announcement_text()
            soup = BeautifulSoup(text, 'lxml')
            context = soup.select('div.col-sm-3:nth-child(3) > div:nth-child(2) > div:nth-child(1) '
                                  '> div:nth-child(2) > div:nth-child(3) > font:nth-child(1)')[0]
            title = context.find('p').find('b').text
            tmp = title.split('，')
            time = ''
            for msg in tmp:
                r = re.search(r'[从|至](.*)(开始|结束)', msg)
                if r:
                    time += r.group(1).strip() + ' - '
            time = time[:-3]
            title = time.split(' - ')[0] + 'UP卡池'
            data['char']['title'] = title
            data['arms']['title'] = title
            data['char']['time'] = time
            data['arms']['time'] = time
            start_idx = -1
            end_idx = -1
            index = 0
            divs = context.find_all('div')
            for x in divs:
                if x.text == '角色':
                    start_idx = index
                if x.text == '武器':
                    end_idx = index
                    break
                index += 1
            for x in divs[start_idx+1: end_idx]:
                name = x.find('p').find_all('a')[-1].text
                data['char']['up_char']['3'][name] = '0'
            for x in divs[end_idx+1:]:
                name = x.find('p').find_all('a')[-1].text
                data['arms']['up_char']['5'][name] = '0'
        except TimeoutError:
            print(f'更新坎公骑冠剑UP池信息超时...')
            with open(pretty_up_char, 'r', encoding='utf8') as f:
                data = json.load(f)
        except Exception as e:
            print(f'坎公骑冠剑up更新失败 {type(e)}：{e}')
        return check_write(data, guardian_up_char, 'guardian')











from nonebot import on_regex, require, on_keyword
from nonebot.adapters.cqhttp import Bot, MessageEvent, Message
from nonebot.permission import SUPERUSER
from nonebot.typing import T_State
from .genshin_handle import genshin_draw, update_genshin_info, reset_count, reload_genshin_pool
from .prts_handle import update_prts_info, prts_draw, reload_prts_pool
from .pretty_handle import update_pretty_info, pretty_draw, reload_pretty_pool
from .guardian_handle import update_guardian_info, guardian_draw
from .pcr_handle import update_pcr_info, pcr_draw
from .azur_handle import update_azur_info, azur_draw
from .fgo_handle import update_fgo_info, fgo_draw
from .onmyoji_handle import update_onmyoji_info, onmyoji_draw
from .update_game_info import update_info
from .util import is_number, check_num
from .rule import is_switch
from .config import PRTS_FLAG, PRETTY_FLAG, GUARDIAN_FLAG, GENSHIN_FLAG, PCR_FLAG, AZUR_FLAG, FGO_FLAG, ONMYOJI_FLAG
from .async_update_game_info import async_update_game
import re
import asyncio

scheduler = require('nonebot_plugin_apscheduler').scheduler

prts = on_regex(r'.*?方舟[1-9|一][0-9]{0,2}[抽|井]', rule=is_switch('prts'), priority=5, block=True)
prts_update = on_keyword({'更新方舟信息', '更新明日方舟信息'}, permission=SUPERUSER, priority=1, block=True)
prts_up_reload = on_keyword({'重载方舟卡池'}, priority=1, block=True)

genshin = on_regex('.*?原神(武器|角色)?池?[1-9|一][0-9]{0,2}[抽|井]', rule=is_switch('genshin'), priority=5, block=True)
genshin_update = on_keyword({'更新原神信息'}, permission=SUPERUSER, priority=1, block=True)
genshin_reset = on_keyword({'重置原神抽卡'}, priority=1, block=True)
genshin_up_reload = on_keyword({'重载原神卡池'}, priority=1, block=True)

pretty = on_regex('.*?马娘卡?[1-9|一][0-9]{0,2}[抽|井]', rule=is_switch('pretty'), priority=5, block=True)
pretty_update = on_keyword({'更新马娘信息', '更新赛马娘信息'}, permission=SUPERUSER, priority=1, block=True)
pretty_up_reload = on_keyword({'重载赛马娘卡池'}, priority=1, block=True)

guardian = on_regex('.*?坎公骑冠剑武?器?[1-9|一][0-9]{0,2}[抽|井]', rule=is_switch('guardian'), priority=5, block=True)
guardian_update = on_keyword({'更新坎公骑冠剑信息'}, permission=SUPERUSER, priority=1, block=True)

pcr = on_regex('.*?(pcr|公主连结|公主连接|公主链接|公主焊接)[1-9|一][0-9]{0,2}[抽|井]', rule=is_switch('pcr'), priority=5, block=True)
pcr_update = on_keyword({'更新pcr信息', '更新公主连结信息'}, permission=SUPERUSER, priority=1, block=True)

azur = on_regex('.*?碧蓝航?线?(轻型|重型|特型)池?[1-9|一][0-9]{0,2}[抽]', rule=is_switch('azur'), priority=5, block=True)
azur_update = on_keyword({'更新碧蓝信息', '更新碧蓝航线信息'}, permission=SUPERUSER, priority=1, block=True)

fgo = on_regex('.*?fgo[1-9|一][0-9]{0,2}[抽]', rule=is_switch('fgo'), priority=5, block=True)
fgo_update = on_keyword({'更新fgo信息'}, permission=SUPERUSER, priority=1, block=True)

onmyoji = on_regex('.*?阴阳师[1-9|一][0-9]{0,2}[抽]', rule=is_switch('onmyoji'), priority=5, block=True)
onmyoji_update = on_keyword({'更新阴阳师信息'}, permission=SUPERUSER, priority=1, block=True)


@prts.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    msg = str(event.get_message()).strip()
    if msg in ['方舟一井', '方舟1井']:
        num = 300
    else:
        rmsg = re.search(r'.*?方舟(.*)抽', msg)
        if rmsg:
            num, flag = check_num(rmsg.group(1), 300)
            if not flag:
                await prts.finish(num, at_sender=True)
        else:
            return
    await prts.send(await prts_draw(int(num)), at_sender=True)


@prts_up_reload.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    text = await reload_prts_pool()
    await prts_up_reload.finish(Message(f'重载完成！\n{text}'))


@genshin.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    msg = str(event.get_message()).strip()
    rmsg = re.search(r'.*?原神(武器|角色)?池?(.*)[抽|井]', msg)
    if rmsg:
        pool_name = rmsg.group(1)
        if pool_name == '武器':
            pool_name = 'arms'
        elif pool_name == '角色':
            pool_name = 'char'
        else:
            pool_name = ''
        num = rmsg.group(2)
        if msg.find('一井') != -1 or msg.find('1井') != -1:
            num = 180
        else:
            num, flag = check_num(num, 180)
            if not flag:
                await genshin.finish(num, at_sender=True)
    else:
        return
    await genshin.send(await genshin_draw(event.user_id, int(num), pool_name), at_sender=True)


@genshin_up_reload.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    text = await reload_genshin_pool()
    await genshin_reset.finish(Message(f'重载成功！\n{text}'))


@genshin_reset.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    reset_count(event.user_id)
    await genshin_reset.send('重置了原神抽卡次数', at_sender=True)


@pretty.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    msg = str(event.get_message()).strip()
    if msg.find('1井') != -1 or msg.find('一井') != -1:
        num = 200
        if msg.find("卡") == -1:
            pool_name = 'char'
        else:
            pool_name = 'card'
    else:
        rmsg = re.search(r'.*?马娘(.*)抽', msg)
        if rmsg:
            num = rmsg.group(1)
            if num[0] == '卡':
                num = num[1:]
                pool_name = 'card'
            else:
                pool_name = 'char'
            num, flag = check_num(num, 200)
            if not flag:
                await pretty.finish(num, at_sender=True)
        else:
            return
    await pretty.send(await pretty_draw(int(num), pool_name), at_sender=True)


@pretty_up_reload.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    text = await reload_pretty_pool()
    await genshin_reset.finish(Message(f'重载成功！\n{text}'))


@guardian.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    msg = str(event.get_message()).strip()
    pool_name = 'char'
    if msg.find('1井') != -1 or msg.find('一井') != -1:
        num = 300
        if msg.find('武器') != -1:
            pool_name = 'arms'
    else:
        rmsg = re.search(r'.*?坎公骑冠剑(.*)抽', msg)
        if rmsg:
            num = rmsg.group(1)
            if num.find('武器') != -1:
                pool_name = 'arms'
                num = num.replace('武器', '')
            num, flag = check_num(num, 300)
            if not flag:
                await guardian.finish(num, at_sender=True)
        else:
            return
    await guardian.send(await guardian_draw(int(num), pool_name), at_sender=True)


@pcr.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    msg = str(event.get_message()).strip()
    if msg.find('1井') != -1 or msg.find('一井') != -1:
        num = 300
    else:
        rmsg = re.search(r'.*?(pcr|公主连结)(.*)[抽|井]', msg)
        if rmsg:
            num, flag = check_num(rmsg.group(2), 300)
            if not flag:
                await pcr.finish(num, at_sender=True)
        else:
            return
    await pcr.send(await pcr_draw(int(num)), at_sender=True)


@azur.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    msg = str(event.get_message()).strip()
    rmsg = re.search('.*?碧蓝航?线?(轻型|重型|特型)池?(.*)[抽]', msg)
    if rmsg:
        pool_name = rmsg.group(1)
        num, flag = check_num(rmsg.group(2), 300)
        if not flag:
            await azur.finish(num, at_sender=True)
    else:
        return
    await azur.send(await azur_draw(int(num), pool_name), at_sender=True)


@fgo.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    msg = str(event.get_message()).strip()
    rmsg = re.search('.*?fgo(.*)抽', msg)
    if rmsg:
        num, flag = check_num(rmsg.group(1), 300)
        if not flag:
            await fgo.finish(num, at_sender=True)
    else:
        return
    await fgo.send(await fgo_draw(int(num)), at_sender=True)


@onmyoji.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    msg = str(event.get_message()).strip()
    rmsg = re.search('.*?阴阳师(.*)抽', msg)
    if rmsg:
        num, flag = check_num(rmsg.group(1), 300)
        if not flag:
            await onmyoji.finish(num, at_sender=True)
    else:
        return
    await onmyoji.send(await onmyoji_draw(int(num)), at_sender=True)


@prts_update.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    await update_prts_info()
    await prts_update.finish('更新完成！')


@genshin_update.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    await update_genshin_info()
    await genshin_update.finish('更新完成！')


@pretty_update.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    await update_pretty_info()
    await genshin_update.finish('更新完成！')


@guardian_update.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    await update_guardian_info()
    await genshin_update.finish('更新完成！')


@pcr_update.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    await update_pcr_info()
    await genshin_update.finish('更新完成！')


@azur_update.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    await update_azur_info()
    await genshin_update.finish('更新完成！')


@fgo_update.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    await update_fgo_info()
    await genshin_update.finish('更新完成！')


@onmyoji_update.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    await update_onmyoji_info()
    await genshin_update.finish('更新完成！')


# 更新资源
@scheduler.scheduled_job(
    'cron',
    hour=4,
    minute=1,
)
async def _():
    tasks = []
    if PRTS_FLAG:
        tasks.append(asyncio.ensure_future(update_prts_info()))
    if GENSHIN_FLAG:
        tasks.append(asyncio.ensure_future(update_genshin_info()))
    if PRETTY_FLAG:
        tasks.append(asyncio.ensure_future(update_pretty_info()))
    if GUARDIAN_FLAG:
        tasks.append(asyncio.ensure_future(update_guardian_info()))
    if PCR_FLAG:
        tasks.append(asyncio.ensure_future(update_pcr_info()))
    if AZUR_FLAG:
        tasks.append(asyncio.ensure_future(update_azur_info()))
    if FGO_FLAG:
        tasks.append(asyncio.ensure_future(update_fgo_info()))
    if ONMYOJI_FLAG:
        tasks.append(asyncio.ensure_future(update_onmyoji_info()))
    await asyncio.gather(*tasks)


# 每天四点重载方舟up卡池
@scheduler.scheduled_job(
    'cron',
    hour=4,
    minute=1,
)
async def _():
    if PRTS_FLAG:
        await reload_prts_pool()


# 每天四点重载赛马娘up卡池
@scheduler.scheduled_job(
    'cron',
    hour=4,
    minute=1,
)
async def _():
    if PRETTY_FLAG:
        await reload_pretty_pool()


# 每天下午六点点重载原神up卡池
@scheduler.scheduled_job(
    'cron',
    hour=18,
    minute=1,
)
async def _():
    if PRTS_FLAG:
        await reload_genshin_pool()





from nonebot.rule import Rule
from nonebot.adapters.cqhttp import Bot, MessageEvent
from nonebot.typing import T_State
from .config import GENSHIN_FLAG, PRTS_FLAG, PRETTY_FLAG, GUARDIAN_FLAG, PCR_FLAG, AZUR_FLAG, FGO_FLAG, ONMYOJI_FLAG


def is_switch(game_name: str) -> Rule:

    async def _is_switch(bot: Bot, event: MessageEvent, state: T_State) -> bool:
        if game_name == 'prts':
            return PRTS_FLAG
        if game_name == 'genshin':
            return GENSHIN_FLAG
        if game_name == 'pretty':
            return PRETTY_FLAG
        if game_name == 'guardian':
            return GUARDIAN_FLAG
        if game_name == 'pcr':
            return PCR_FLAG
        if game_name == 'azur':
            return AZUR_FLAG
        if game_name == 'fgo':
            return FGO_FLAG
        if game_name == 'onmyoji':
            return ONMYOJI_FLAG
        else:
            return False

    return Rule(_is_switch)

















import nonebot
from nonebot.adapters.cqhttp import MessageSegment
from .announcement import PrettyAnnouncement
from .update_game_info import update_info
from .util import init_star_rst, generate_img, max_card, BaseData, \
    set_list, get_star, format_card_information, UpEvent
import random
from .config import PRETTY_THREE_P, PRETTY_TWO_P, DRAW_PATH, PRETTY_ONE_P, PRETTY_FLAG
from dataclasses import dataclass
from .init_card_pool import init_game_pool

try:
    import ujson as json
except ModuleNotFoundError:
    import json

driver: nonebot.Driver = nonebot.get_driver()

ALL_CHAR = []
ALL_CARD = []

_CURRENT_CHAR_POOL_TITLE = ""
_CURRENT_CARD_POOL_TITLE = ""
UP_CHAR = []
UP_CARD = []
POOL_IMG = []


@dataclass
class PrettyChar(BaseData):
    pass


async def pretty_draw(count: int, pool_name):
    if pool_name == 'card':
        cnlist = ['SSR', 'SR', 'R']
    else:
        cnlist = ['★★★', '★★', '★']
    star_list = [0, 0, 0]
    obj_list, obj_dict, three_list, star_list, three_olist = format_card_information(count, star_list,
                                                                                     _get_pretty_card, pool_name)
    up_type = []
    up_list = []
    title = ''
    if pool_name == 'char' and _CURRENT_CHAR_POOL_TITLE:
        up_type = UP_CHAR
        title = _CURRENT_CHAR_POOL_TITLE
    elif pool_name == 'card' and _CURRENT_CARD_POOL_TITLE:
        up_type = UP_CARD
        title = _CURRENT_CARD_POOL_TITLE
    tmp = ''
    if up_type:
        for x in up_type:
            for operator in x.operators:
                up_list.append(operator.split(']')[1] if pool_name == 'char' else operator)
            if x.star == 3:
                if pool_name == 'char':
                    tmp += f'三星UP：{" ".join(x.operators)} \n'
                else:
                    tmp += f'SSR UP：{" ".join(x.operators)} \n'
            elif x.star == 2:
                if pool_name == 'char':
                    tmp += f'二星UP：{" ".join(x.operators)} \n'
                else:
                    tmp += f'SR UP：{" ".join(x.operators)} \n'
            elif x.star == 1:
                if pool_name == 'char':
                    tmp += f'一星UP：{" ".join(x.operators)} '
                else:
                    tmp += f'R UP：{" ".join(x.operators)} '
    tmp = tmp[:-1] if tmp and tmp[-1] == '\n' else tmp
    pool_info = f'当前up池：{title}\n{tmp}' if title else ''
    rst = init_star_rst(star_list, cnlist, three_list, three_olist, up_list)
    if count > 90:
        obj_list = set_list(obj_list)
    return pool_info + MessageSegment.image(
        "base64://" + await generate_img(obj_list, 'pretty', star_list)) \
           + '\n' + rst[:-1] + '\n' + max_card(obj_dict)


async def update_pretty_info():
    global ALL_CHAR, ALL_CARD
    url = 'https://wiki.biligame.com/umamusume/赛马娘图鉴'
    data, code = await update_info(url, 'pretty')
    if code == 200:
        ALL_CHAR = init_game_pool('pretty', data, PrettyChar)
    url = 'https://wiki.biligame.com/umamusume/支援卡图鉴'
    data, code = await update_info(url, 'pretty_card')
    if code == 200:
        ALL_CARD = init_game_pool('pretty_card', data, PrettyChar)
        await _init_up_char()


async def init_pretty_data():
    global ALL_CHAR, ALL_CARD
    if PRETTY_FLAG:
        with open(DRAW_PATH + 'pretty.json', 'r', encoding='utf8') as f:
            pretty_char_dict = json.load(f)
        with open(DRAW_PATH + 'pretty_card.json', 'r', encoding='utf8') as f:
            pretty_card_dict = json.load(f)
        ALL_CHAR = init_game_pool('pretty', pretty_char_dict, PrettyChar)
        ALL_CARD = init_game_pool('pretty_card', pretty_card_dict, PrettyChar)
        await _init_up_char()


# 抽取卡池
def _get_pretty_card(pool_name: str):
    global ALL_CHAR, ALL_CARD, _CURRENT_CHAR_POOL_TITLE, _CURRENT_CARD_POOL_TITLE
    star = get_star([3, 2, 1], [PRETTY_THREE_P, PRETTY_TWO_P, PRETTY_ONE_P])
    if pool_name == 'card':
        title = _CURRENT_CARD_POOL_TITLE
        up_data = UP_CARD
        data = ALL_CARD
    else:
        title = _CURRENT_CHAR_POOL_TITLE
        up_data = UP_CHAR
        data = ALL_CHAR
    # 有UP池子
    if title and star in [x.star for x in up_data]:
        all_char_lst = [x for x in data if x.star == star and not x.limited]
        # 抽到UP
        if random.random() < 1 / len(all_char_lst) * (0.7 / 0.1385):
            all_up_star = [x.operators for x in up_data if x.star == star][0]
            acquire_operator = random.choice(all_up_star)
            if pool_name == 'char':
                acquire_operator = acquire_operator.split(']')[1]
            acquire_operator = [x for x in data if x.name == acquire_operator][0]
        else:
            acquire_operator = random.choice([x for x in data if x.star == star and not x.limited])
    else:
        acquire_operator = random.choice([x for x in data if x.star == star and not x.limited])
    return acquire_operator, 3 - star


# 获取up和概率
async def _init_up_char():
    global _CURRENT_CHAR_POOL_TITLE, _CURRENT_CARD_POOL_TITLE, UP_CHAR, UP_CARD, POOL_IMG
    UP_CHAR = []
    UP_CARD = []
    up_char_dict = await PrettyAnnouncement.update_up_char()
    _CURRENT_CHAR_POOL_TITLE = up_char_dict['char']['title']
    _CURRENT_CARD_POOL_TITLE = up_char_dict['card']['title']
    if _CURRENT_CHAR_POOL_TITLE and _CURRENT_CARD_POOL_TITLE:
        POOL_IMG = MessageSegment.image(up_char_dict['char']['pool_img']) + \
                   MessageSegment.image(up_char_dict['card']['pool_img'])
    print(f'成功获取赛马娘当前up信息...当前up池: {_CURRENT_CHAR_POOL_TITLE} & {_CURRENT_CARD_POOL_TITLE}')
    for key in up_char_dict.keys():
        for star in up_char_dict[key]['up_char'].keys():
            up_lst = []
            for char in up_char_dict[key]['up_char'][star].keys():
                up_lst.append(char)
            if key == 'char':
                if up_lst:
                    UP_CHAR.append(UpEvent(star=int(star), operators=up_lst, zoom=0))
            else:
                if up_lst:
                    UP_CARD.append(UpEvent(star=int(star), operators=up_lst, zoom=0))


async def reload_pretty_pool():
    await _init_up_char()
    return f'当前UP池子：{_CURRENT_CHAR_POOL_TITLE} & {_CURRENT_CARD_POOL_TITLE} {POOL_IMG}'





from nonebot.adapters.cqhttp import MessageSegment
import random
from .update_game_simple_info import update_simple_info
from .util import generate_img, init_star_rst, BaseData, set_list, get_star, max_card, format_card_information
from .config import AZUR_ONE_P, AZUR_TWO_P, AZUR_THREE_P, AZUR_FOUR_P, AZUR_FLAG, DRAW_PATH
from dataclasses import dataclass
from .init_card_pool import init_game_pool
try:
    import ujson as json
except ModuleNotFoundError:
    import json

ALL_CHAR = []


@dataclass
class AzurChar(BaseData):
    itype: str      # 舰娘类型


async def azur_draw(count: int, pool_name: str):
    #            0      1      2
    cnlist = ['金', '紫', '蓝', '白']
    star_list = [0, 0, 0, 0]
    char_list, char_dict, max_star_list, star_list, max_star_index_list = \
        format_card_information(count, star_list, _get_azur_card, pool_name)
    rst = init_star_rst(star_list, cnlist, max_star_list, max_star_index_list)
    if count > 90:
        char_list = set_list(char_list)
    return MessageSegment.image("base64://" + await generate_img(char_list, 'azur', star_list)) \
           + '\n' + rst[:-1] + '\n' + max_card(char_dict)


async def update_azur_info():
    global ALL_CHAR
    url = 'https://wiki.biligame.com/blhx/舰娘图鉴'
    data, code = await update_simple_info(url, 'azur')
    if code == 200:
        ALL_CHAR = init_game_pool('azur', data, AzurChar)


async def init_azur_data():
    global ALL_CHAR
    if AZUR_FLAG:
        with open(DRAW_PATH + 'azur.json', 'r', encoding='utf8') as f:
            azur_dict = json.load(f)
        ALL_CHAR = init_game_pool('azur', azur_dict, AzurChar)


# 抽取卡池
def _get_azur_card(pool_name: str):
    global ALL_CHAR
    if pool_name == '轻型':
        itype = ['驱逐', '轻巡', '维修']
    elif pool_name == '重型':
        itype = ['重巡', '战列', '战巡', '重炮']
    else:
        itype = ['维修', '潜艇', '重巡', '轻航', '航母']
    star = get_star([4, 3, 2, 1], [AZUR_FOUR_P, AZUR_THREE_P, AZUR_TWO_P, AZUR_ONE_P])
    chars = [x for x in ALL_CHAR if x.star == star and x.itype in itype and not x.limited]
    return random.choice(chars), 4 - star





import os
import nonebot
from nonebot.adapters.cqhttp import MessageSegment
from .update_game_info import update_info
from .util import init_star_rst, generate_img, max_card, BaseData,\
    set_list, get_star, format_card_information, UpEvent
import random
from .config import DRAW_PATH, GUARDIAN_ONE_CHAR_P, GUARDIAN_TWO_CHAR_P, GUARDIAN_THREE_CHAR_P, \
    GUARDIAN_THREE_CHAR_UP_P, GUARDIAN_TWO_ARMS_P, GUARDIAN_FIVE_ARMS_P, GUARDIAN_THREE_CHAR_OTHER_P, \
    GUARDIAN_FOUR_ARMS_P, GUARDIAN_THREE_ARMS_P, GUARDIAN_EXCLUSIVE_ARMS_P, GUARDIAN_EXCLUSIVE_ARMS_UP_P, \
    GUARDIAN_EXCLUSIVE_ARMS_OTHER_P, GUARDIAN_FLAG
from dataclasses import dataclass
from .init_card_pool import init_game_pool
from .announcement import GuardianAnnouncement
try:
    import ujson as json
except ModuleNotFoundError:
    import json

driver: nonebot.Driver = nonebot.get_driver()

ALL_CHAR = []
ALL_ARMS = []

_CURRENT_CHAR_POOL_TITLE = ''
_CURRENT_ARMS_POOL_TITLE = ''
UP_CHAR = []
UP_ARMS = []
POOL_IMG = ''


@dataclass
class GuardianChar(BaseData):
    pass


@dataclass
class GuardianArms(BaseData):
    pass


async def guardian_draw(count: int, pool_name):
    if pool_name == 'arms':
        cnlist = ['★★★★★', '★★★★', '★★★', '★★']
        star_list = [0, 0, 0, 0]
    else:
        cnlist = ['★★★', '★★', '★']
        star_list = [0, 0, 0]
    title = ''
    up_type = []
    up_list = []
    if pool_name == 'char' and _CURRENT_CHAR_POOL_TITLE:
        up_type = UP_CHAR
        title = _CURRENT_CHAR_POOL_TITLE
    elif pool_name == 'arms' and _CURRENT_ARMS_POOL_TITLE:
        up_type = UP_ARMS
        title = _CURRENT_ARMS_POOL_TITLE
    tmp = ''
    if up_type:
        for x in up_type:
            for operator in x.operators:
                up_list.append(operator)
            if pool_name == 'char':
                if x.star == 3:
                    tmp += f'三星UP：{" ".join(x.operators)} \n'
            else:
                if x.star == 5:
                    tmp += f'五星UP：{" ".join(x.operators)}'
    obj_list, obj_dict, max_list, star_list, max_index_list = format_card_information(count, star_list,
                                                                                      _get_guardian_card, pool_name)
    rst = init_star_rst(star_list, cnlist, max_list, max_index_list, up_list)
    pool_info = f'当前up池：{title}\n{tmp}' if title else ''
    if count > 90:
        obj_list = set_list(obj_list)
    return pool_info + '\n' + MessageSegment.image(
        "base64://" + await generate_img(obj_list, 'guardian', star_list)) \
           + '\n' + rst[:-1] + '\n' + max_card(obj_dict)


async def update_guardian_info():
    global ALL_CHAR, ALL_ARMS
    url = 'https://wiki.biligame.com/gt/英雄筛选表'
    data, code = await update_info(url, 'guardian')
    if code == 200:
        ALL_CHAR = init_game_pool('guardian', data, GuardianChar)
    url = 'https://wiki.biligame.com/gt/武器'
    tmp, code_1 = await update_info(url, 'guardian_arms')
    url = 'https://wiki.biligame.com/gt/盾牌'
    data, code_2 = await update_info(url, 'guardian_arms')
    if code_1 == 200 and code_2 == 200:
        data.update(tmp)
        ALL_ARMS = init_game_pool('guardian_arms', data, GuardianArms)
    await _init_up_char()


async def init_guardian_data():
    global ALL_CHAR, ALL_ARMS
    if GUARDIAN_FLAG:
        if not os.path.exists(DRAW_PATH + 'guardian.json') or not os.path.exists(DRAW_PATH + 'guardian_arms.json'):
            await update_guardian_info()
        else:
            with open(DRAW_PATH + 'guardian.json', 'r', encoding='utf8') as f:
                guardian_char_dict = json.load(f)
            with open(DRAW_PATH + 'guardian_arms.json', 'r', encoding='utf8') as f:
                guardian_arms_dict = json.load(f)
            ALL_CHAR = init_game_pool('guardian', guardian_char_dict, GuardianChar)
            ALL_ARMS = init_game_pool('guardian_arms', guardian_arms_dict, GuardianArms)
        await _init_up_char()


# 抽取卡池
def _get_guardian_card(pool_name: str = ''):
    global ALL_ARMS, ALL_CHAR, UP_ARMS, UP_CHAR, _CURRENT_ARMS_POOL_TITLE, _CURRENT_CHAR_POOL_TITLE
    if pool_name == 'char':
        star = get_star([3, 2, 1], [GUARDIAN_THREE_CHAR_P, GUARDIAN_TWO_CHAR_P, GUARDIAN_ONE_CHAR_P])
        up_lst = UP_CHAR
        flag = _CURRENT_CHAR_POOL_TITLE
        _max_star = 3
        all_data = ALL_CHAR
    else:
        star = get_star([5, 4, 3, 2], [GUARDIAN_FIVE_ARMS_P, GUARDIAN_FOUR_ARMS_P,
                                       GUARDIAN_THREE_ARMS_P, GUARDIAN_TWO_ARMS_P])
        up_lst = UP_ARMS
        flag = _CURRENT_ARMS_POOL_TITLE
        _max_star = 5
        all_data = ALL_ARMS
    # 是否UP
    if flag and star == _max_star and pool_name:
        # 获取up角色列表
        up_char_lst = [x.operators for x in up_lst if x.star == star][0]
        # 成功获取up角色
        if random.random() < 0.5:
            up_char_name = random.choice(up_char_lst)
            acquire_char = [x for x in all_data if x.name == up_char_name][0]
        else:
            # 无up
            all_char_lst = [x for x in all_data if x.star == star and x.name not in up_char_lst and not x.limited]
            acquire_char = random.choice(all_char_lst)
    else:
        chars = [x for x in all_data if x.star == star and not x.limited]
        acquire_char = random.choice(chars)
    return acquire_char, _max_star - star


# 获取up和概率
async def _init_up_char():
    global _CURRENT_CHAR_POOL_TITLE, _CURRENT_ARMS_POOL_TITLE, UP_CHAR, UP_ARMS, POOL_IMG
    UP_CHAR = []
    UP_ARMS = []
    up_char_dict = await GuardianAnnouncement.update_up_char()
    _CURRENT_CHAR_POOL_TITLE = up_char_dict['char']['title']
    _CURRENT_ARMS_POOL_TITLE = up_char_dict['arms']['title']
    if _CURRENT_CHAR_POOL_TITLE and _CURRENT_ARMS_POOL_TITLE:
        POOL_IMG = MessageSegment.image(up_char_dict['char']['pool_img']) + \
                   MessageSegment.image(up_char_dict['arms']['pool_img'])
    print(f'成功获取坎公骑冠剑当前up信息...当前up池: {_CURRENT_CHAR_POOL_TITLE} & {_CURRENT_ARMS_POOL_TITLE}')
    for key in up_char_dict.keys():
        for star in up_char_dict[key]['up_char'].keys():
            up_char_lst = []
            for char in up_char_dict[key]['up_char'][star].keys():
                up_char_lst.append(char)
            if key == 'char':
                UP_CHAR.append(UpEvent(star=int(star), operators=up_char_lst, zoom=0))
            else:
                UP_ARMS.append(UpEvent(star=int(star), operators=up_char_lst, zoom=0))


async def reload_guardian_pool():
    await _init_up_char()
    return f'当前UP池子：{_CURRENT_CHAR_POOL_TITLE} & {_CURRENT_ARMS_POOL_TITLE} {POOL_IMG}'






import platform
import aiohttp
import aiofiles
from asyncio.exceptions import TimeoutError
from aiohttp.client_exceptions import InvalidURL
from typing import List, Union, Set
from pathlib import Path
from .config import path_dict, DRAW_PATH
import nonebot
import pypinyin
from PIL import UnidentifiedImageError
from .create_img import CreateImg
import random
from dataclasses import dataclass
import os
import asyncio
try:
    import ujson as json
except ModuleNotFoundError:
    import json


driver: nonebot.Driver = nonebot.get_driver()


loop = asyncio.get_event_loop()


headers = {'User-Agent': '"Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; TencentTraveler 4.0)"'}


@dataclass
class BaseData:
    name: str
    star: int
    limited: bool  # 限定


@dataclass
class UpEvent:
    star: int  # 对应up星级
    operators: List[BaseData]  # 干员列表
    zoom: float  # up提升倍率


async def download_img(url: str, path: str, name: str) -> bool:
    path = path.split('_')[0]
    codename = cn2py(name)
    if not os.path.exists(DRAW_PATH + f'/draw_card/{path}/{codename}.png'):
        try:
            async with aiohttp.ClientSession(headers=headers) as session:
                async with session.get(url, timeout=7) as response:
                    async with aiofiles.open(DRAW_PATH + f'/draw_card/{path}/{codename}.png', 'wb') as f:
                        await f.write(await response.read())
                        print(f'下载 {path_dict[path]} 图片成功，名称：{name}，url：{url}')
                        return True
        except TimeoutError:
            print(f'下载 {path_dict[path]} 图片超时，名称：{name}，url：{url}')
            return False
        except InvalidURL:
            print(f'下载 {path_dict[path]} 链接错误，名称：{name}，url：{url}')
            return False
    else:
        # logger.info(f'{path_dict[path]} 图片 {name} 已存在')
        return False


@driver.on_startup
def _check_dir():
    for dir_name in path_dict.keys():
        _p = Path(DRAW_PATH + f'/draw_card/' + dir_name)
        if not _p.exists():
            _p.mkdir(parents=True, exist_ok=True)


async def generate_img(card_set: Union[Set[BaseData], List[BaseData]], game_name: str, star_list: list) -> str:
    # try:
    img_list = []
    background_list = []
    for x in card_set:
        if game_name == 'prts':
            if x.star == 6:
                background_list.append('#FFD700')
            elif x.star == 5:
                background_list.append('#DAA520')
            elif x.star == 4:
                background_list.append('#9370D8')
            else:
                background_list.append('white')
        if game_name == 'azur':
            if os.path.exists(DRAW_PATH + f'/draw_card/{game_name}/{x.star}_star.png'):
                background_list.append(DRAW_PATH + f'/draw_card/{game_name}/{x.star}_star.png')
        pyname = cn2py(x.name)
        img_list.append(DRAW_PATH + f'/draw_card/{game_name}/{pyname}.png')
    img_len = len(img_list)
    w = 100 * 10
    if img_len <= 10:
        w = 100 * img_len
        h = 100
    elif img_len % 10 == 0:
        h = 100 * int(img_len / 10)
    else:
        h = 100 * int(img_len / 10) + 100
    card_img = await asyncio.get_event_loop().run_in_executor(None, _pst, h, img_list, game_name, background_list)
    num = 0
    for n in star_list:
        num += n
    A = CreateImg(w, h)
    A.paste(card_img)
    return A.pic2bs4()


def _pst(h: int, img_list: list, game_name: str, background_list: list):
    card_img = CreateImg(100 * 10, h, 100, 100)
    idx = 0
    for img in img_list:
        try:
            if game_name == 'prts':
                bk = CreateImg(100, 100, color=background_list[idx])
                b = CreateImg(94, 94, background=img)
                bk.paste(b, (3, 3))
                b = bk
            elif game_name == 'azur' and background_list:
                bk = CreateImg(100, 100, background=background_list[idx])
                b = CreateImg(98, 90, background=img)
                bk.paste(b, (1, 5))
                b = bk
            else:
                try:
                    b = CreateImg(100, 100, background=img)
                except UnidentifiedImageError as e:
                    print(f'无法识别图片 已删除图片，下次更新重新下载... e：{e}')
                    if os.path.exists(img):
                        os.remove(img)
                    b = CreateImg(100, 100, color='black')
        except FileNotFoundError:
            print(f'{img} not exists')
            b = CreateImg(100, 100, color='black')
        card_img.paste(b)
        idx += 1
    return card_img


def init_star_rst(star_list: list, cnlist: list, max_star_list: list, max_star_index_list: list, up_list: list = None) -> str:
    if not up_list:
        up_list = []
    rst = ''
    for i in range(len(star_list)):
        if star_list[i]:
            rst += f'[{cnlist[i]}×{star_list[i]}] '
    rst += '\n'
    for i in range(len(max_star_list)):
        if max_star_list[i] in up_list:
            rst += f'第 {max_star_index_list[i]+1} 抽获取UP {max_star_list[i]}\n'
        else:
            rst += f'第 {max_star_index_list[i]+1} 抽获取 {max_star_list[i]}\n'
    return rst


def max_card(_dict: dict):
    _max_value = max(_dict.values())
    _max_user = list(_dict.keys())[list(_dict.values()).index(_max_value)]
    return f'抽取到最多的是{_max_user}，共抽取了{_max_value}次'
    # ThreeHighest = nlargest(3, operator_dict, key=operator_dict.get)
    # rst = '最喜欢你的前三位是干员是：\n'
    # for name in ThreeHighest:
    #     rst += f'{name} 共投了 {operator_dict[name]} 份简历\n'
    # return rst[:-1]


def is_number(s) -> bool:
    try:
        float(s)
        return True
    except ValueError:
        pass
    try:
        import unicodedata
        unicodedata.numeric(s)
        return True
    except (TypeError, ValueError):
        pass
    return False


def cn2py(word) -> str:
    temp = ""
    for i in pypinyin.pinyin(word, style=pypinyin.NORMAL):
        temp += ''.join(i)
    return temp


def set_list(lst: List[BaseData]) -> list:
    tmp = []
    name_lst = []
    for x in lst:
        if x.name not in name_lst:
            tmp.append(x)
            name_lst.append(x.name)
    return tmp


def get_star(star_lst: List[int], probability_lst: List[float]) -> int:
    rand = random.random()
    add = 0
    tmp_lst = [(0, probability_lst[0])]
    for i in range(1, len(probability_lst) - 1):
        add += probability_lst[i - 1]
        tmp_lst.append((tmp_lst[i - 1][1], probability_lst[i] + add))
    tmp_lst.append((tmp_lst[-1][1], 1))
    for i in range(len(tmp_lst)):
        if tmp_lst[i][0] <= rand <= tmp_lst[i][1]:
            return star_lst[i]


# 整理数据
def format_card_information(count: int, star_list: List[int], func, pool_name: str = ''):
    max_star_lst = []       # 获取的最高星级角色列表
    max_index_lst = []      # 获取最高星级角色的次数
    obj_list = []           # 获取所有角色
    obj_dict = {}           # 获取角色次数字典
    for i in range(count):
        if pool_name:
            obj, code = func(pool_name)
        else:
            obj, code = func()
        star_list[code] += 1
        if code == 0:
            max_star_lst.append(obj.name)
            max_index_lst.append(i)
        try:
            obj_dict[obj.name] += 1
        except KeyError:
            obj_dict[obj.name] = 1
        obj_list.append(obj)
    return obj_list, obj_dict, max_star_lst, star_list, max_index_lst


# 检测次数是否合法
def check_num(num: str, max_num: int) -> 'str, bool':
    if is_number(num):
        try:
            num = int(num)
        except ValueError:
            return '必！须！是！数！字！', False
    if num > max_num:
        return '一井都满不足不了你嘛！快爬开！', False
    if num < 1:
        return '虚空抽卡？？？', False
    else:
        return str(num), True


# 移除windows和linux下特殊字符
def remove_prohibited_str(name: str):
    if platform.system().lower() == 'windows':
        tmp = ''
        for i in name:
            if i not in ['\\', '/', ':', '*', '?', '"', '<', '>', '|']:
                tmp += i
        name = tmp
    else:
        name = name.replace('/', '\\')
    return name




from setuptools import setup

setup(name='nonebot-plugin-gamedraw',
      version='0.1.7.7',
      description='Nonebot2插件',
      url='https://github.com/HibiKier/nonebot_plugin_gamedraw',
      author='hibikier',
      author_email='775757368@qq.com',
      license='MIT',
      packages=['nonebot_plugin_gamedraw'])



import ujson as json
from nonebot.adapters.cqhttp import MessageSegment
import nonebot
import random
from .update_game_info import update_info
from .update_game_simple_info import update_simple_info
from .util import generate_img, init_star_rst, BaseData, set_list, get_star, max_card
from .config import PCR_TWO_P, PCR_THREE_P, PCR_ONE_P, DRAW_PATH, PCR_FLAG, PCR_G_TWO_P, PCR_G_THREE_P, PCR_TAI
from dataclasses import dataclass
from .init_card_pool import init_game_pool

driver: nonebot.Driver = nonebot.get_driver()

ALL_CHAR = []


@dataclass
class PcrChar(BaseData):
    pass


async def pcr_draw(count: int):
    #            0      1      2
    cnlist = ['★★★', '★★', '★']
    char_list, three_list, three_index_list, char_dict, star_list = _format_card_information(count)
    rst = init_star_rst(star_list, cnlist, three_list, three_index_list)
    if count > 90:
        char_list = set_list(char_list)
    return MessageSegment.image("base64://" + await generate_img(char_list, 'pcr', star_list)) \
           + '\n' + rst[:-1] + '\n' + max_card(char_dict)


async def update_pcr_info():
    global ALL_CHAR
    if PCR_TAI:
        url = 'https://wiki.biligame.com/pcr/角色图鉴'
        data, code = await update_simple_info(url, 'pcr')
    else:
        url = 'https://wiki.biligame.com/pcr/角色筛选表'
        data, code = await update_info(url, 'pcr')
    if code == 200:
        ALL_CHAR = init_game_pool('pcr', data, PcrChar)


async def init_pcr_data():
    global ALL_CHAR
    if PCR_FLAG:
        with open(DRAW_PATH + 'pcr.json', 'r', encoding='utf8') as f:
            pcr_dict = json.load(f)
        ALL_CHAR = init_game_pool('pcr', pcr_dict, PcrChar)


# 抽取卡池
def _get_pcr_card(mode: int = 1):
    global ALL_CHAR
    if mode == 2:
        star = get_star([3, 2], [PCR_G_THREE_P, PCR_G_TWO_P])
    else:
        star = get_star([3, 2, 1], [PCR_THREE_P, PCR_TWO_P, PCR_ONE_P])
    chars = [x for x in ALL_CHAR if x.star == star and not x.limited]
    return random.choice(chars), 3 - star


def _format_card_information(_count: int):
    char_list = []
    star_list = [0, 0, 0]
    three_index_list = []
    three_list = []
    char_dict = {}
    # 保底计算
    count = 0
    for i in range(_count):
        count += 1
        # 十连保底
        if count == 10:
            char, code = _get_pcr_card(2)
            count = 0
        else:
            char, code = _get_pcr_card()
            if code < 2:
                count = 0
        star_list[code] += 1
        if code == 0:
            three_list.append(char.name)
            three_index_list.append(i)
        try:
            char_dict[char.name] += 1
        except KeyError:
            char_dict[char.name] = 1
        char_list.append(char)
    return char_list, three_list, three_index_list, char_dict, star_list




from nonebot.adapters.cqhttp import MessageSegment
import random
from .update_game_requests_info import update_requests_info
from .util import generate_img, init_star_rst, BaseData, set_list, get_star, max_card
from .config import ONMYOJI_SR, ONMYOJI_SSR, ONMYOJI_SP, ONMYOJI_R, DRAW_PATH, ONMYOJI_FLAG
from dataclasses import dataclass
from .init_card_pool import init_game_pool
try:
    import ujson as json
except ModuleNotFoundError:
    import json

ALL_CHAR = []


@dataclass
class OnmyojiChar(BaseData):
    pass


async def onmyoji_draw(count: int):
    #            0      1      2
    cnlist = ['SP', 'SSR', 'SR', 'R']
    obj_list, obj_dict, star_list, rst = format_card_information(count)
    rst = init_star_rst(star_list, cnlist, [], []) + rst
    if count > 90:
        obj_list = set_list(obj_list)
    return MessageSegment.image("base64://" + await generate_img(obj_list, 'onmyoji', star_list)) \
           + '\n' + rst[:-1] + '\n' + max_card(obj_dict)


async def update_onmyoji_info():
    global ALL_CHAR
    data, code = await update_requests_info('onmyoji')
    if code == 200:
        ALL_CHAR = init_game_pool('onmyoji', data, OnmyojiChar)


async def init_onmyoji_data():
    global ALL_CHAR
    if ONMYOJI_FLAG:
        with open(DRAW_PATH + 'onmyoji.json', 'r', encoding='utf8') as f:
            azur_dict = json.load(f)
        ALL_CHAR = init_game_pool('onmyoji', azur_dict, OnmyojiChar)


onmyoji_star = {
    5: 'SP',
    4: 'SSR',
    3: 'SR',
    2: 'R',
}


# 抽取卡池
def _get_onmyoji_card():
    global ALL_CHAR
    star = get_star([5, 4, 3, 2], [ONMYOJI_SP, ONMYOJI_SSR, ONMYOJI_SR, ONMYOJI_R])
    chars = [x for x in ALL_CHAR if x.star == onmyoji_star[star] and not x.limited]
    return random.choice(chars), 5 - star


def format_card_information(count: int):
    star_list = [0, 0, 0, 0]
    obj_list = []           # 获取所有角色
    obj_dict = {}           # 获取角色次数字典
    rst = ''
    for i in range(count):
        obj, code = _get_onmyoji_card()
        star_list[code] += 1
        if code == 0:
            rst += f'第 {i+1} 抽获取SP {obj.name}\n'
        elif code == 1:
            rst += f'第 {i+1} 抽获取SSR {obj.name}\n'
        try:
            obj_dict[obj.name] += 1
        except KeyError:
            obj_dict[obj.name] = 1
        obj_list.append(obj)
    return obj_list, obj_dict, star_list, rst




import os
from nonebot.adapters.cqhttp import MessageSegment
import nonebot
import random
from .update_game_info import update_info
from .util import generate_img, init_star_rst, BaseData, set_list, get_star, UpEvent
from .config import GENSHIN_FIVE_P, GENSHIN_FOUR_P, GENSHIN_G_FIVE_P, GENSHIN_G_FOUR_P, GENSHIN_THREE_P, I72_ADD, \
    DRAW_PATH, GENSHIN_FLAG
from dataclasses import dataclass
from .init_card_pool import init_game_pool
from .announcement import GenshinAnnouncement
try:
    import ujson as json
except ModuleNotFoundError:
    import json

driver: nonebot.Driver = nonebot.get_driver()

genshin_five = {}
genshin_count = {}
genshin_pl_count = {}

ALL_CHAR = []
ALL_ARMS = []

UP_CHAR = []
UP_ARMS = []


_CURRENT_CHAR_POOL_TITLE = ''
_CURRENT_ARMS_POOL_TITLE = ''
POOL_IMG = ''


@dataclass
class GenshinChar(BaseData):
    pass


async def genshin_draw(user_id: int, count: int, pool_name: str):
    #                   0      1      2
    cnlist = ['★★★★★', '★★★★', '★★★']
    char_list, five_list, five_index_list, char_dict, star_list = _format_card_information(count, user_id, pool_name)
    temp = ''
    title = ''
    up_type = []
    up_list = []
    if pool_name == 'char' and _CURRENT_CHAR_POOL_TITLE:
        up_type = UP_CHAR
        title = _CURRENT_CHAR_POOL_TITLE
    elif pool_name == 'arms' and _CURRENT_ARMS_POOL_TITLE:
        up_type = UP_ARMS
        title = _CURRENT_ARMS_POOL_TITLE
    tmp = ''
    if up_type:
        for x in up_type:
            for operator in x.operators:
                up_list.append(operator)
            if x.star == 5:
                tmp += f'五星UP：{" ".join(x.operators)} \n'
            elif x.star == 4:
                tmp += f'四星UP：{" ".join(x.operators)}'
    rst = init_star_rst(star_list, cnlist, five_list, five_index_list, up_list)
    pool_info = f'当前up池：{title}\n{tmp}' if title else ''
    if count > 90:
        char_list = set_list(char_list)
    return pool_info + '\n' + MessageSegment.image("base64://" + await generate_img(char_list, 'genshin', star_list)) + '\n' + rst[:-1] + \
           temp[:-1] + f'\n距离保底发还剩 {90 - genshin_count[user_id] if genshin_count.get(user_id) else "^"} 抽' \
           + "\n【五星：0.6%，四星：5.1%\n第72抽开始五星概率每抽加0.585%】"


async def update_genshin_info():
    global ALL_CHAR, ALL_ARMS
    url = 'https://wiki.biligame.com/ys/角色筛选'
    data, code = await update_info(url, 'genshin')
    if code == 200:
        ALL_CHAR = init_game_pool('genshin', data, GenshinChar)
    url = 'https://wiki.biligame.com/ys/武器图鉴'
    data, code = await update_info(url, 'genshin_arms', ['头像', '名称', '类型', '稀有度.alt',
                                                         '获取途径', '初始基础属性1', '初始基础属性2',
                                                         '攻击力（MAX）', '副属性（MAX）', '技能'])
    if code == 200:
        ALL_ARMS = init_game_pool('genshin_arms', data, GenshinChar)
    await _init_up_char()


async def init_genshin_data():
    global ALL_CHAR, ALL_ARMS
    if GENSHIN_FLAG:
        if not os.path.exists(DRAW_PATH + 'genshin.json') or not os.path.exists(DRAW_PATH + 'genshin_arms.json'):
            await update_genshin_info()
        else:
            with open(DRAW_PATH + 'genshin.json', 'r', encoding='utf8') as f:
                genshin_dict = json.load(f)
            with open(DRAW_PATH + 'genshin_arms.json', 'r', encoding='utf8') as f:
                genshin_ARMS_dict = json.load(f)
            ALL_CHAR = init_game_pool('genshin', genshin_dict, GenshinChar)
            ALL_ARMS = init_game_pool('genshin_arms', genshin_ARMS_dict, GenshinChar)
        await _init_up_char()


# 抽取卡池
def _get_genshin_card(mode: int = 1, pool_name: str = '', add: float = 0.0):
    global ALL_ARMS, ALL_CHAR, UP_ARMS, UP_CHAR, _CURRENT_ARMS_POOL_TITLE, _CURRENT_CHAR_POOL_TITLE
    if mode == 1:
        star = get_star([5, 4, 3], [GENSHIN_FIVE_P + add, GENSHIN_FOUR_P, GENSHIN_THREE_P])
    elif mode == 2:
        star = get_star([5, 4], [GENSHIN_G_FIVE_P + add, GENSHIN_G_FOUR_P])
    else:
        star = 5
    if pool_name == 'char':
        data_lst = UP_CHAR
        flag = _CURRENT_CHAR_POOL_TITLE
        itype_all_lst = ALL_CHAR + [x for x in ALL_ARMS if x.star == star and x.star < 5]
    elif pool_name == 'arms':
        data_lst = UP_ARMS
        flag = _CURRENT_ARMS_POOL_TITLE
        itype_all_lst = ALL_ARMS + [x for x in ALL_CHAR if x.star == star and x.star < 5]
    else:
        data_lst = ''
        flag = ''
        itype_all_lst = ''
    all_lst = ALL_ARMS + ALL_CHAR
    # 是否UP
    if flag and star > 3 and pool_name:
        # 获取up角色列表
        up_char_lst = [x.operators for x in data_lst if x.star == star][0]
        # 成功获取up角色
        if random.random() < 0.5:
            up_char_name = random.choice(up_char_lst)
            acquire_char = [x for x in all_lst if x.name == up_char_name][0]
        else:
            # 无up
            all_char_lst = [x for x in itype_all_lst if x.star == star and x.name not in up_char_lst and not x.limited]
            acquire_char = random.choice(all_char_lst)
    else:
        chars = [x for x in all_lst if x.star == star and not x.limited]
        acquire_char = random.choice(chars)
    return acquire_char, 5 - star


def _format_card_information(_count: int, user_id, pool_name):
    char_list = []
    star_list = [0, 0, 0]
    five_index_list = []
    five_list = []
    five_dict = {}
    add = 0.0
    if genshin_count.get(user_id) and _count <= 90:
        f_count = genshin_count[user_id]
    else:
        f_count = 0
    if genshin_pl_count.get(user_id) and _count <= 90:
        count = genshin_pl_count[user_id]
    else:
        count = 0
    for i in range(_count):
        count += 1
        f_count += 1
        # 十连保底
        if count == 10 and f_count != 90:
            if f_count >= 72:
                add += I72_ADD
            char, code = _get_genshin_card(2, pool_name, add=add)
            count = 0
        # 大保底
        elif f_count == 90:
            char, code = _get_genshin_card(3, pool_name)
        else:
            if f_count >= 72:
                add += I72_ADD
            char, code = _get_genshin_card(pool_name=pool_name, add=add)
            if code == 1:
                count = 0
        star_list[code] += 1
        if code == 0:
            if _count <= 90:
                genshin_five[user_id] = f_count
            add = 0.0
            f_count = 0
            five_list.append(char.name)
            five_index_list.append(i)
            try:
                five_dict[char.name] += 1
            except KeyError:
                five_dict[char.name] = 1
        char_list.append(char)
    if _count <= 90:
        genshin_count[user_id] = f_count
        genshin_pl_count[user_id] = count
    return char_list, five_list, five_index_list, five_dict, star_list


def reset_count(user_id: int):
    genshin_count[user_id] = 0
    genshin_pl_count[user_id] = 0


# 获取up和概率
async def _init_up_char():
    global _CURRENT_CHAR_POOL_TITLE, _CURRENT_ARMS_POOL_TITLE, UP_CHAR, UP_ARMS, POOL_IMG
    UP_CHAR = []
    UP_ARMS = []
    up_char_dict = await GenshinAnnouncement.update_up_char()
    _CURRENT_CHAR_POOL_TITLE = up_char_dict['char']['title']
    _CURRENT_ARMS_POOL_TITLE = up_char_dict['arms']['title']
    if _CURRENT_CHAR_POOL_TITLE and _CURRENT_ARMS_POOL_TITLE:
        POOL_IMG = MessageSegment.image(up_char_dict['char']['pool_img']) + \
                   MessageSegment.image(up_char_dict['arms']['pool_img'])
    print(f'成功获取原神当前up信息...当前up池: {_CURRENT_CHAR_POOL_TITLE} & {_CURRENT_ARMS_POOL_TITLE}')
    for key in up_char_dict.keys():
        for star in up_char_dict[key]['up_char'].keys():
            up_char_lst = []
            for char in up_char_dict[key]['up_char'][star].keys():
                up_char_lst.append(char)
            if key == 'char':
                UP_CHAR.append(UpEvent(star=int(star), operators=up_char_lst, zoom=0))
            else:
                UP_ARMS.append(UpEvent(star=int(star), operators=up_char_lst, zoom=0))


async def reload_genshin_pool():
    await _init_up_char()
    return f'当前UP池子：{_CURRENT_CHAR_POOL_TITLE} & {_CURRENT_ARMS_POOL_TITLE} {POOL_IMG}'







import asyncio
import nonebot
import os
from .pcr_handle import update_pcr_info, init_pcr_data
from .azur_handle import update_azur_info, init_azur_data
from .prts_handle import update_prts_info, init_prts_data
from .pretty_handle import update_pretty_info, init_pretty_data
from .guardian_handle import update_guardian_info, init_guardian_data
from .genshin_handle import update_genshin_info, init_genshin_data
from .fgo_handle import update_fgo_info, init_fgo_data
from .onmyoji_handle import update_onmyoji_info, init_onmyoji_data
from .config import DRAW_PATH, PRTS_FLAG, PRETTY_FLAG, GUARDIAN_FLAG, PCR_FLAG, AZUR_FLAG, GENSHIN_FLAG, FGO_FLAG, \
    ONMYOJI_FLAG


driver: nonebot.Driver = nonebot.get_driver()


@driver.on_startup
async def async_update_game():
    tasks = []
    init_lst = [init_pcr_data, init_pretty_data, init_azur_data, init_prts_data, init_genshin_data, init_guardian_data,
                init_fgo_data, init_onmyoji_data]
    if PRTS_FLAG and not os.path.exists(DRAW_PATH + 'prts.json'):
        tasks.append(asyncio.ensure_future(update_prts_info()))
        init_lst.remove(init_prts_data)

    if PRETTY_FLAG and (not os.path.exists(DRAW_PATH + 'pretty.json') or
                        not os.path.exists(DRAW_PATH + 'pretty_card.json')):
        tasks.append(asyncio.ensure_future(update_pretty_info()))
        init_lst.remove(init_pretty_data)

    if GUARDIAN_FLAG and not os.path.exists(DRAW_PATH + 'guardian.json'):
        tasks.append(asyncio.ensure_future(update_guardian_info()))

    if PCR_FLAG and not os.path.exists(DRAW_PATH + 'pcr.json'):
        tasks.append(asyncio.ensure_future(update_pcr_info()))
        init_lst.remove(init_pcr_data)

    if GENSHIN_FLAG and (not os.path.exists(DRAW_PATH + 'genshin.json') or
                         not os.path.exists(DRAW_PATH + 'genshin_arms.json')):
        tasks.append(asyncio.ensure_future(update_genshin_info()))
        init_lst.remove(init_genshin_data)

    if AZUR_FLAG and not os.path.exists(DRAW_PATH + 'azur.json'):
        tasks.append(asyncio.ensure_future(update_azur_info()))
        init_lst.remove(init_azur_data)

    if FGO_FLAG and (not os.path.exists(DRAW_PATH + 'fgo.json') or
                     not os.path.exists(DRAW_PATH + 'fgo_card.json')):
        tasks.append(asyncio.ensure_future(update_fgo_info()))
        init_lst.remove(init_fgo_data)

    if ONMYOJI_FLAG and not os.path.exists(DRAW_PATH + 'onmyoji.json'):
        tasks.append(asyncio.ensure_future(update_onmyoji_info()))
        init_lst.remove(init_onmyoji_data)
    try:
        await asyncio.gather(*tasks)
        for func in init_lst:
            await func()
    except asyncio.exceptions.CancelledError:
        print('更新异常：CancelledError，再次更新...')
        await async_update_game()








from PIL import Image, ImageDraw
import base64
from io import BytesIO


class CreateImg:
    def __init__(self,
                 w,
                 h,
                 img_w=0,
                 img_h=0,
                 color='white',
                 image_type='RGBA',
                 background='',
                 divisor=1):
        self.w = int(w)
        self.h = int(h)
        self.img_w = int(img_w)
        self.img_h = int(img_h)
        self.current_w = 0
        self.current_h = 0
        if not background:
            self.markImg = Image.new(image_type, (self.w, self.h), color)
        else:
            if w == 0 and h == 0:
                self.markImg = Image.open(background)
                w, h = self.markImg.size
                if divisor:
                    self.w = int(divisor * w)
                    self.h = int(divisor * h)
                    self.markImg = self.markImg.resize((self.w, self.h), Image.ANTIALIAS)
                else:
                    self.w = w
                    self.h = h
            else:
                self.markImg = Image.open(background).resize((self.w, self.h), Image.ANTIALIAS)
        self.draw = ImageDraw.Draw(self.markImg)
        self.size = self.w, self.h

    # 贴图
    def paste(self, img, pos=None, alpha=False):
        if isinstance(img, CreateImg):
            img = img.markImg
        if self.current_w == self.w:
            self.current_w = 0
            self.current_h += self.img_h
        if not pos:
            pos = (self.current_w, self.current_h)
        if alpha:
            try:
                self.markImg.paste(img, pos, img)
            except ValueError:
                img = img.convert("RGBA")
                self.markImg.paste(img, pos, img)
        else:
            self.markImg.paste(img, pos)
        self.current_w += self.img_w
        return self.markImg

    # 转bs4:
    def pic2bs4(self):
        buf = BytesIO()
        self.markImg.save(buf, format='PNG')
        base64_str = base64.b64encode(buf.getvalue()).decode()
        return base64_str


